@@ -1,4 +1,5 @@
-# -*- coding: utf-8 -*-
+# emacs: -*- coding: utf-8; mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
+# vi: set ft=python sts=4 ts=4 sw=4 et:
 #
 # sampledoc documentation build configuration file, created by
 # sphinx-quickstart on Tue Jun  3 12:40:24 2008.
@@ -18,6 +19,11 @@ import sys, os
 # absolute, like shown here.
 sys.path.append(os.path.abspath('sphinxext'))
 
+# Get project related strings.  Please do not change this line to use
+# execfile because execfile is not available in Python 3
+_info_fname = os.path.join('..', 'nipy', 'info.py')
+rel = {}
+exec(open(_info_fname, 'rt').read(), {}, rel)
 
 # Import support for ipython console session syntax highlighting (lives
 # in the sphinxext directory defined above)
@@ -30,30 +36,16 @@ import ipython_console_highlighting
 # coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
 extensions = ['sphinx.ext.autodoc',
               'sphinx.ext.doctest',
-              'ipython_console_highlighting', 
-              'inheritance_diagram', 
-              'numpydoc',
-              'autosummary']
+              'sphinx.ext.pngmath',
+              'sphinx.ext.autosummary',
+              'inheritance_diagram',
+              'numpy_ext.numpydoc',
+              'matplotlib.sphinxext.plot_directive',
+              'matplotlib.sphinxext.only_directives', # needed for above
+              ]
 
-# Matplotlib sphinx extensions
-# ----------------------------
-
-# Currently we depend on some matplotlib extentions that are only in
-# the trunk, so we've added copies of these files to fall back on,
-# since most people install releases.  Once theses extensions have
-# been released for a while we should remove this hack.  I'm assuming
-# any modifications to these extensions will be done upstream in
-# matplotlib!  The matplotlib trunk will have more bug fixes and
-# feature updates so we'll try to use that one first.
-try:
-    import matplotlib.sphinxext
-    extensions.append('matplotlib.sphinxext.mathmpl')
-    extensions.append('matplotlib.sphinxext.only_directives')
-    extensions.append('matplotlib.sphinxext.plot_directive')
-except ImportError:
-    extensions.append('mathmpl')
-    extensions.append('only_directives')
-    extensions.append('plot_directive')
+# Autosummary on
+autosummary_generate=True
 
 # Add any paths that contain templates here, relative to this directory.
 templates_path = ['_templates']
@@ -62,20 +54,21 @@ templates_path = ['_templates']
 source_suffix = '.rst'
 
 # The master toctree document.
-master_doc = 'contents'
+master_doc = 'index'
 
 # General substitutions.
 project = 'nipy'
-#copyright = ':ref:`2005-2009, Neuroimaging in Python team. <nipy-software-license>`' 
-copyright = '2005-2009, Neuroimaging in Python team' 
+
+#copyright = ':ref:`2005-2010, Neuroimaging in Python team. <nipy-software-license>`'
+copyright = '2005-2012, Neuroimaging in Python team'
 
 # The default replacements for |version| and |release|, also used in various
 # other places throughout the built documents.
 #
 # The short X.Y version.
-version = '0.1'
+version = rel['__version__']
 # The full version, including alpha/beta/rc tags.
-release = '0.1'
+release = version
 
 # There are two options for replacing |today|: either, you set today to some
 # non-false value, then it is used:
@@ -88,7 +81,7 @@ unused_docs = []
 
 # List of directories, relative to source directories, that shouldn't
 # be searched for source files.
-exclude_trees = ['www']
+# exclude_trees = []
 
 # what to put into API doc (just class doc, just init, or both)
 autoclass_content = 'class'
@@ -110,6 +103,10 @@ pygments_style = 'sphinx'
 
 # Options for HTML output
 # -----------------------
+#
+# The theme to use for HTML and HTML Help pages.  Major themes that come with
+# Sphinx are currently 'default' and 'sphinxdoc'.
+html_theme = 'sphinxdoc'
 
 # The style sheet to use for HTML and HTML Help pages. A file of that name
 # must exist either in Sphinx' static/ path, or in one of the custom paths
@@ -138,10 +135,10 @@ html_last_updated_fmt = '%b %d, %Y'
 #html_use_smartypants = True
 
 # Content template for the index page.
-html_index = 'www/index.html'
+html_index = 'index.html'
 
 # Custom sidebar templates, maps document names to template names.
-html_sidebars = {'index': 'indexsidebar.html'}
+# html_sidebars = {'index': 'indexsidebar.html'}
 
 # Additional templates that should be rendered to pages, maps page names to
 # template names.
@@ -164,7 +161,6 @@ html_copy_source = True
 # Output file base name for HTML help builder.
 htmlhelp_basename = project
 
-
 # Options for LaTeX output
 # ------------------------
 
@@ -179,15 +175,9 @@ htmlhelp_basename = project
 # [howto/manual]).
 
 latex_documents = [
-  ('contents', 'nipy.tex', 'Neuroimaging in Python Documentation',
-   ur'The Neuroimaging in Python documentation team.','manual'),
-  ## ('devel/index', 'nipy_devel.tex',
-  ##  'Neuroimaging in Python Developer Documentation',
-  ##  ur'The Neuroimaging in Python documentation team.','manual'),
-  ## ('api/index', 'nipy_ref.tex',
-  ##  'Neuroimaging in Python Reference Guide',
-  ##  ur'The Neuroimaging in Python documentation team.','manual'),
-]
+  ('documentation', 'nipy.tex', 'Neuroimaging in Python Documentation',
+   ur'Neuroimaging in Python team.','manual'),
+  ]
 
 # The name of an image file (relative to this directory) to place at the top of
 # the title page.
