@@ -5,21 +5,21 @@ This plot demonstrates a neuronal model that is a sum
 of delta functions times coefficient values
 """
 
-import pylab
-from matplotlib import rc
-rc('text', usetex=True)
+import matplotlib.pyplot as plt
 
+# Coefficients for a and b
 ba = 1
 bb = -2
 
-ta = [0,4,8,12,16]; tb = [2,6,10,14,18]
+# Times for a and b
+ta = [0,4,8,12,16]
+tb = [2,6,10,14,18]
 
 for t in ta:
-    pylab.plot([t,t],[0,ba],c='r')
+    plt.plot([t,t],[0,ba],c='r')
 for t in tb:
-    pylab.plot([t,t],[0,bb],c='b')
+    plt.plot([t,t],[0,bb],c='b')
 
-a = pylab.gca()
+a = plt.gca()
 a.set_xlabel(r'$t$')
 a.set_ylabel(r'$n(t)$')
-
