@@ -5,7 +5,6 @@ One-dimensional Gamma-Gaussian mixture density classes : Given a set
 of points the algo provides approcumate maximum likelihood estimates
 of the mixture distribution using an EM algorithm.
 
-
 Author: Bertrand Thirion and Merlin Keller 2005-2008
 """
 
@@ -75,8 +74,8 @@ def _gaus_dens(mean, var, x):
 def _gam_dens(shape, scale, x):
     """evaluate the gamma density (shape,scale) at points x
 
-    Note
-    ----
+    Notes
+    -----
     Returns 0 on negative subspace
     """
     ng = np.zeros(np.size(x))
@@ -96,8 +95,8 @@ def _gam_param(x, z):
     x: array of shape(nbitem) the learning points
     z: array of shape(nbitem), their membership within the class
 
-    Note
-    ----
+    Notes
+    -----
     if no point is positive then the couple (1, 1) is returned
     """
     eps = 1.e-5
@@ -118,8 +117,8 @@ def _gam_param(x, z):
 
 
 class Gamma(object):
-    """
-    This is the basic one dimensional Gaussian-Gamma Mixture estimation class
+    """ Basic one dimensional Gaussian-Gamma Mixture estimation class
+
     Note that it can work with positive or negative values,
     as long as there is at least one positive value.
     NB : The gamma distribution is defined only on positive values.
@@ -195,10 +194,12 @@ class GGM(object):
         Mstep of the model: maximum likelihood
         estimation of the parameters of the model
 
-        Parameters:
-        -----------
-        x array of shape(nbitems), input data
-        z array of shape(nbitrems,2): the membership matrix
+        Parameters
+        ----------
+        x  : array of shape (nbitems,)
+            input data
+        z array of shape(nbitrems, 2)
+            the membership matrix
         """
         # z[0,:] is the likelihood to be generated by the gamma
         # z[1,:] is the likelihood to be generated by the gaussian
@@ -223,7 +224,7 @@ class GGM(object):
 
         Returns
         -------
-        z: array of shape (nbitrems, 2)
+        z: array of shape (nbitems, 2)
             the membership matrix
         """
         eps = 1.e-15
@@ -241,13 +242,19 @@ class GGM(object):
 
         Parameters
         ----------
-        x: array of shape(nbitems): the data to be processed
-        niter = 100:  max nb of iterations
-        delta = 0.001: criterion for convergence
+        x : array of shape (nbitems,)
+            the data to be processed
+        niter : int, optional
+            max nb of iterations
+        delta : float, optional
+            criterion for convergence
+        verbose : bool, optional
+            If True, print values during iterations
 
         Returns
         -------
-        LL, float, average final log-likelihood
+        LL, float
+            average final log-likelihood
         """
         if x.max() < 0:
             # all the values are generated by the Gaussian
@@ -271,11 +278,12 @@ class GGM(object):
         return L
 
     def show(self, x):
-        """ vizualisation of the mm based on the empirical histogram of x
+        """ Visualization of the mm based on the empirical histogram of x
 
         Parameters
         ----------
-        x: array of shape(nbitems): the data to be processed
+        x : array of shape (nbitems,)
+            the data to be processed
         """
         step = 3.5 * np.std(x) / np.exp(np.log(np.size(x)) / 3)
         bins = max(10, int((x.max() - x.min()) / step))
@@ -302,12 +310,12 @@ class GGM(object):
         """Posterior probability of observing the data x for each component
 
         Parameters
-        -------------
+        ----------
         x: array of shape (nbitems,)
             the data to be processed
 
         Returns
-        ---------
+        -------
         y, pg : arrays of shape (nbitem)
             the posterior probability
         """
@@ -345,13 +353,16 @@ class GGGM(object):
 
         Parameters
         -----------
-        shape_n=1, scale_n=1: float,
+        shape_n : float, optional
+        scale_n: float, optional
             parameters of the nehative gamma; must be positive
-        mean=0, var=1: float,
+        mean : float, optional
+        var : float, optional
             parameters of the gaussian ; var must be positive
-        shape_p=1, scale_p=1: float,
+        shape_p : float, optional
+        scale_p : float, optional
             parameters of the positive gamma; must be positive
-        mixt: array of shape(3), optional,
+        mixt : array of shape (3,), optional
             the mixing proportions; they should be positive and sum to 1
         """
         self.shape_n = shape_n
@@ -378,13 +389,12 @@ class GGGM(object):
         """
         initialization of the differnt parameters
 
-        Parameters:
-        -------------
+        Parameters
+        ----------
         x: array of shape(nbitems)
            the data to be processed
-        mixt=None, array of shape(3), optional
-                  prior mixing proportions
-        if mixt==None, the classes have equal weight
+        mixt : None or array of shape(3), optional
+           prior mixing proportions. If None, the classes have equal weight
         """
         if mixt != None:
             if np.size(mixt) == 3:
@@ -428,7 +438,7 @@ class GGGM(object):
         nothing more that the tails of the distribution.
 
         Parameters
-        -----------
+        ----------
         x: array of shape(nbitem)
             the data under consideration
         dof: integer, optional
@@ -443,14 +453,14 @@ class GGGM(object):
             x = x.copy()
         # positive gamma
         i = np.ravel(np.nonzero(x > 0))
-        from ..statistics.empirical_pvalue import FDR
+        from ..statistics.empirical_pvalue import fdr
 
         if np.size(i) > 0:
             if dof < 0:
                 pvals = st.norm.sf(x)
             else:
                 pvals = st.t.sf(x, dof)
-            q = FDR().all_fdr(pvals)
+            q = fdr(pvals)
             z = 1 - q[i]
             self.mixt[2] = np.maximum(0.5, z.sum()) / np.size(x)
             self.shape_p, self.scale_p = _gam_param(x[i], z)
@@ -464,7 +474,7 @@ class GGGM(object):
                 pvals = st.norm.cdf(x)
             else:
                 pvals = st.t.cdf(x, dof)
-            q = FDR().all_fdr(pvals)
+            q = fdr(pvals)
             z = 1 - q[i]
             self.shape_n, self.scale_n = _gam_param( - x[i], z)
             self.mixt[0] = np.maximum(0.5, z.sum()) / np.size(x)
@@ -502,17 +512,17 @@ class GGGM(object):
         """ Update probabilistic memberships of the three components
 
         Parameters
-        -------------
+        ----------
         x: array of shape (nbitems,)
             the input data
 
         Returns
-        ----------
+        -------
         z: ndarray of shape (nbitems, 3)
             probabilistic membership
 
         Notes
-        ------
+        -----
         z[0,:] is the membership the negative gamma
         z[1,:] is the membership of  the gaussian
         z[2,:] is the membership of the positive gamma
@@ -529,7 +539,7 @@ class GGGM(object):
         """ Whole EM estimation procedure:
 
         Parameters
-        ------------
+        ----------
         x: array of shape (nbitem)
             input data
         niter: integer, optional
@@ -545,7 +555,7 @@ class GGGM(object):
             verbosity level
 
         Returns
-        ---------
+        -------
         z: array of shape (nbitem, 3)
             the membership matrix
         """
@@ -590,8 +600,8 @@ class GGGM(object):
         ng,y,pg: three arrays of shape(nbitem)
             the posteriori of the 3 components given the data
 
-        Note
-        ----
+        Notes
+        -----
         ng + y + pg = np.ones(nbitem)
         """
         p = self.mixt
@@ -621,12 +631,10 @@ class GGGM(object):
         return ng, y, pg
 
     def show(self, x, mpaxes=None):
-        """
-        Vizualization of the mixture, superimposed on the empirical
-        histogram of x
+        """ Visualization of mixture shown on the empirical histogram of x
 
         Parameters
-        -----------
+        ----------
         x: ndarray of shape (nditem,)
             data
         mpaxes: matplotlib axes, optional
