@@ -26,8 +26,8 @@ Author : Bertrand Thirion,Pamela Guevara, 2006-2009
 
 import numpy as np
 
-from ..graph.graph import WeightedGraph 
-from ..graph.forest import Forest 
+from ..graph.graph import WeightedGraph
+from ..graph.forest import Forest
 
 
 class WeightedForest(Forest):
@@ -231,168 +231,6 @@ class WeightedForest(Forest):
 
         return lst[n:self.V]
 
-    def plot2(self, addNodes=False, font_size=10, cl_size=None):
-        """
-        Plot the dendrogram associated with self
-        """
-        if self.check_compatible_height() == False:
-            raise ValueError('cannot plot myself in my current state')
-        n = np.sum(self.isleaf())
-
-        # 1. find a permutation of the leaves that makes it nice
-        aux = _label2(self.parents, self.children)
-        rank = aux
-
-        # 2. derive the abscissa in the dendrogram
-        idx = - np.ones(self.V)
-        temp2 = np.argsort(rank[:self.V])
-        isleaf = self.isleaf()
-        temp = np.zeros(n)
-        notleaf = np.zeros(self.V - n)
-        nleaf = []
-
-        cont = 0
-        cont2 = 0
-        for i in xrange(np.size(rank)):
-            ind = temp2[i]
-            if isleaf[ind]:
-                temp[cont] = temp2[i]
-                cont = cont + 1
-            else:
-                notleaf[cont2] = temp2[i]
-                nleaf.append(int(temp2[i]))
-                cont2 = cont2 + 1
-
-        for i in range(n):
-            idx[temp[i]] = i
-
-        while len(nleaf) > 0:
-            for i in nleaf:
-                ch = self.children[i]
-                idxch = idx[ch]
-                test = (idxch == -1)
-                ok = True
-                for t in test:
-                    if t == True:
-                        ok = False
-                if ok:
-                    idx[i] = np.mean(idxch)
-                    nleaf.remove(i)
-
-        # 3. plot
-        import matplotlib.pylab as mp
-        mp.figure()
-
-        for i in range(self.V):
-            h1 = self.height[i]
-            h2 = self.height[self.parents[i]]
-            mp.plot([idx[i], idx[i]], [h1, h2], 'k')
-
-        ch = self.get_children()
-        for i in range(self.V):
-            if np.size(ch[i]) > 0:
-                lidx = idx[ch[i]]
-                m = lidx.min()
-                M = lidx.max()
-                h = self.height[i]
-                mp.plot([m, M], [h, h], 'k')
-
-        cM = 1.05 * self.height.max() - 0.05 * self.height.min()
-        cm = 1.05 * self.height.min() - 0.05 * self.height.max()
-        mp.axis([-1, idx.max() + 1, cm, cM])
-
-        if addNodes:
-            for i in range(self.V):
-                h1 = self.height[i]
-                mp.text(idx[i] + 0.05, h1 + 0.45, str(i), fontsize=font_size,
-                                     color='b')
-
-        if cl_size != None:
-            for i in range(self.V):
-                h1 = self.height[i]
-                text = str(cl_size[i])
-                mp.text(idx[i] + 0.5, h1 + 0.05, text, fontsize=font_size)
-
-        return rank
-
-    def fancy_plot_(self, valid):
-        """Idem plot, but the valid edges are enahanced
-
-        Returns
-        -------
-        ax, the axes handle of the plot
-        """
-        if self.check_compatible_height() == False:
-            raise ValueError('cannot plot myself in my current state')
-
-        n = np.sum(self.isleaf())
-        # 1. find a permutation of the leaves that makes it nice
-        aux = _label(self.parents)
-        temp = np.zeros(self.V)
-        rank = np.arange(self.V)
-        temp[:n] = np.argsort(aux[:n])
-        for i in range(n):
-            rank[temp[i]] = i
-
-        # 2. derive the abscissa in the dendrogram
-        idx = np.zeros(self.V)
-        temp = np.argsort(rank[:n])
-        for i in range(n):
-            idx[temp[i]] = i
-        for i in range(n, self.V):
-            j = np.nonzero(self.parents == i)[0]
-            idx[i] = np.mean(idx[j])
-
-        # 3. plot
-        import matplotlib.pylab as mp
-        mp.figure()
-        ax = mp.axes()
-        for i in range(self.V):
-            h1 = self.height[i]
-            h2 = self.height[self.parents[i]]
-            mp.plot([idx[i], idx[i]], [h1, h2], 'k')
-            if valid[i] & valid[self.parents[i]]:
-                mp.plot([idx[i], idx[i]], [h1, h2], 'b', linewidth=2)
-
-        ch = self.get_children()
-        for i in range(self.V):
-            if np.size(ch[i]) > 0:
-                chi = ch[i]
-                lidx = idx[chi]
-                j = chi[lidx.argmin()]
-                k = chi[lidx.argmax()]
-                m = lidx.min()
-                M = lidx.max()
-                h = self.height[i]
-                mp.plot([m, M], [h, h], 'k')
-                if valid[j] | valid[k]:
-                    mp.plot([m, M], [h, h], 'b', linewidth=2)
-
-        cM = 1.05 * self.height.max() - 0.05 * self.height.min()
-        cm = 1.05 * self.height.min() - 0.05 * self.height.max()
-        mp.axis([ - 1, idx.max() + 1, cm, cM])
-        return ax
-
-    def fancy_plot(self, validleaves):
-        """Idem plot, but the valid edges are enahanced
-        """
-        if self.check_compatible_height() == False:
-            raise ValueError('cannot plot myself in my current state')
-
-        valid = np.zeros(self.V, 'bool')
-        valid[self.isleaf()] = validleaves.astype('bool')
-        nv = np.sum(validleaves)
-        nv0 = 0
-        while nv > nv0:
-            nv0 = nv
-            for v in range(self.V):
-                if valid[v]:
-                    valid[self.parents[v]] = 1
-            nv = np.sum(valid)
-
-        rank = self.fancy_plot_(valid)
-        return rank
-
 
 #--------------------------------------------------------------------------
 #------------- Average link clustering on a graph -------------------------
@@ -440,7 +278,7 @@ def fusion(K, pop, i, j, k):
             i1 = idxk[acorr[a]]
             i2 = idxk[acorr[a + 1]]
             K.weights[i1] = K.weights[i1] + K.weights[i2]
-            K.weights[i2] = - np.infty
+            K.weights[i2] = - np.inf
             K.edges[i2] = -1
 
     #right side
@@ -453,7 +291,7 @@ def fusion(K, pop, i, j, k):
             i1 = idxk[acorr[a]]
             i2 = idxk[acorr[a + 1]]
             K.weights[i1] = K.weights[i1] + K.weights[i2]
-            K.weights[i2] = - np.infty
+            K.weights[i2] = - np.inf
             K.edges[i2] = - 1
 
 
@@ -476,7 +314,7 @@ def average_link_graph(G):
     e.g. distance-based procedures).  Thus the tree is created with
     negated affinity values, in roder to respect the traditional
     ordering of cluster potentials. individual points have the
-    potential (-np.infty).
+    potential (-np.inf).
     This problem is handled transparently inthe associated segment functionp.
     """
 
@@ -490,7 +328,7 @@ def average_link_graph(G):
 
     parent = np.arange(2 * n - nbcc, dtype=np.int)
     pop = np.ones(2 * n - nbcc, np.int)
-    height = np.infty * np.ones(2 * n - nbcc)
+    height = np.inf * np.ones(2 * n - nbcc)
 
     # iteratively merge clusters
     for q in range(n - nbcc):
@@ -505,10 +343,10 @@ def average_link_graph(G):
 
         # 2. remove the current edge
         K.edges[m] = -1
-        K.weights[m] = - np.infty
+        K.weights[m] = - np.inf
         m = np.nonzero((K.edges[:, 0] == j) * (K.edges[:, 1] == i))[0]
         K.edges[m] = - 1
-        K.weights[m] = - np.infty
+        K.weights[m] = - np.inf
 
         # 3. merge the edges with third part edges
         parent[i] = k
@@ -522,16 +360,18 @@ def average_link_graph(G):
     return t
 
 
-def average_link_graph_segment(G, stop=0, qmax=1, verbose=0):
+def average_link_graph_segment(G, stop=0, qmax=1, verbose=False):
     """Agglomerative function based on a (hopefully sparse) similarity graph
 
     Parameters
     ----------
     G the input graph
-    stop: float, the stopping criterion
-    qmax: int, optional,
-          the number of desired clusters
-          (in the limit of the stopping criterion)
+    stop: float
+        the stopping criterion
+    qmax: int, optional
+        the number of desired clusters (in the limit of the stopping criterion)
+    verbose : bool, optional
+        If True, print diagnostic information
 
     Returns
     -------
@@ -622,7 +462,7 @@ def _initial_inertia(K, Features, seeds=None):
             if (aux[i] or aux[j]):
                 K.weights[e] = _inertia(i, j, Features)
             else:
-                K.weights[e] = np.infty
+                K.weights[e] = np.inf
 
 
 def _auxiliary_graph(G, Features):
@@ -735,7 +575,7 @@ def _remap(K, i, j, k, Features, linc, rinc):
         for a in range(np.size(scorr) - 1):
             if scorr[a] == scorr[a + 1]:
                 i2 = idxk[acorr[a + 1]]
-                K.weights[i2] = np.infty
+                K.weights[i2] = np.inf
                 rinc[K.edges[i2, 1]].remove(i2)
                 K.edges[i2] = - 1
                 linc[k].remove(i2)
@@ -749,33 +589,37 @@ def _remap(K, i, j, k, Features, linc, rinc):
         for a in range(np.size(scorr) - 1):
             if scorr[a] == scorr[a + 1]:
                 i2 = idxk[acorr[a + 1]]
-                K.weights[i2] = np.infty
+                K.weights[i2] = np.inf
                 linc[K.edges[i2, 0]].remove(i2)
                 K.edges[i2] = - 1
                 rinc[k].remove(i2)
     return linc, rinc
 
 
-def ward_quick(G, feature, verbose=0):
+def ward_quick(G, feature, verbose=False):
     """ Agglomerative function based on a topology-defining graph
     and a feature matrix.
 
     Parameters
     ----------
-    G graph instance,
-      topology-defining graph
-    feature: array of shape (G.V,dim_feature):
-            some vectorial information related to the graph vertices
+    G : graph instance
+        topology-defining graph
+    feature: array of shape (G.V,dim_feature)
+        some vectorial information related to the graph vertices
+    verbose : bool, optional
+         If True, print diagnostic information
 
     Returns
     -------
     t: weightForest instance,
        that represents the dendrogram of the data
 
-    Note
+    Notes
     ----
     Hopefully a quicker version
+
     A euclidean distance is used in the feature space
+
     Caveat : only approximate
     """
     # basic check
@@ -805,7 +649,7 @@ def ward_quick(G, feature, verbose=0):
     while (q < n - nbcc):
         # 1. find the lightest edges
         aux = np.zeros(2 * n)
-        ape = np.nonzero(K.weights < np.infty)
+        ape = np.nonzero(K.weights < np.inf)
         ape = np.reshape(ape, np.size(ape))
         idx = np.argsort(K.weights[ape])
 
@@ -830,7 +674,7 @@ def ward_quick(G, feature, verbose=0):
 
             # 2. remove the current edge
             K.edges[m] = -1
-            K.weights[m] = np.infty
+            K.weights[m] = np.inf
             linc[i].remove(m)
             rinc[j].remove(m)
 
@@ -838,7 +682,7 @@ def ward_quick(G, feature, verbose=0):
             if np.sum(K.edges[ml, 1] == i) > 0:
                 m = ml[np.flatnonzero(K.edges[ml, 1] == i)]
                 K.edges[m] = -1
-                K.weights[m] = np.infty
+                K.weights[m] = np.inf
                 linc[j].remove(m)
                 rinc[i].remove(m)
 
@@ -856,16 +700,19 @@ def ward_quick(G, feature, verbose=0):
     return t
 
 
-def ward_field_segment(F, stop=-1, qmax=-1, verbose=0):
+def ward_field_segment(F, stop=-1, qmax=-1, verbose=False):
     """Agglomerative function based on a field structure
 
     Parameters
     ----------
     F the input field (graph+feature)
-    stop: float, the stopping crterion
-         if stop==-1, then no stopping criterion is used
-    qmax: int, the maximum number of desired clusters
-            (in the limit of the stopping criterion)
+    stop: float, optional
+        the stopping crterion.  if stop==-1, then no stopping criterion is used
+    qmax: int, optional
+        the maximum number of desired clusters (in the limit of the stopping
+        criterion)
+    verbose : bool, optional
+        If True, print diagnostic information
 
     Returns
     -------
@@ -874,19 +721,18 @@ def ward_field_segment(F, stop=-1, qmax=-1, verbose=0):
     cost array of shape (F.V - 1)
          the cost of each merge step during the clustering procedure
 
-    CAVEAT
-    ------
-    only approximate
 
-    NOTE
-    ----
-    look ward_quick_segment for more information
+    Notes
+    -----
+    See ward_quick_segment for more information
+
+    Caveat : only approximate
     """
     u, cost = ward_quick_segment(F, F.field, stop, qmax, verbose)
     return u, cost
 
 
-def ward_quick_segment(G, feature, stop=-1, qmax=1, verbose=0):
+def ward_quick_segment(G, feature, stop=-1, qmax=1, verbose=False):
     """
     Agglomerative function based on a topology-defining graph
     and a feature matrix.
@@ -896,11 +742,14 @@ def ward_quick_segment(G, feature, stop=-1, qmax=1, verbose=0):
     G: labs.graph.WeightedGraph instance
        the input graph (a topological graph essentially)
     feature array of shape (G.V,dim_feature)
-            vectorial information related to the graph vertices
-    stop = -1: the stopping crterion
-         if stop==-1, then no stopping criterion is used
-    qmax=1: the maximum number of desired clusters
-            (in the limit of the stopping criterion)
+        vectorial information related to the graph vertices
+    stop1 : int or float, optional
+        the stopping crterion if stop==-1, then no stopping criterion is used
+    qmax : int, optional
+        the maximum number of desired clusters (in the limit of the stopping
+        criterion)
+    verbose : bool, optional
+        If True, print diagnostic information
 
     Returns
     -------
@@ -909,14 +758,13 @@ def ward_quick_segment(G, feature, stop=-1, qmax=1, verbose=0):
     cost: array of shape (G.V - 1)
           the cost of each merge step during the clustering procedure
 
-    NOTE
-    ----
+    Notes
+    -----
     Hopefully a quicker version
+
     A euclidean distance is used in the feature space
 
-    CAVEAT
-    ------
-    only approximate
+    Caveat : only approximate
     """
     # basic check
     if feature.ndim == 1:
@@ -928,7 +776,7 @@ def ward_quick_segment(G, feature, stop=-1, qmax=1, verbose=0):
 
     n = G.V
     if stop == - 1:
-        stop = np.infty
+        stop = np.inf
     qmax = int(np.minimum(qmax, n - 1))
     t = ward_quick(G, feature, verbose)
     if verbose:
@@ -951,20 +799,24 @@ def ward_quick_segment(G, feature, stop=-1, qmax=1, verbose=0):
     return u, cost
 
 
-def ward_segment(G, feature, stop=-1, qmax=1, verbose=0):
+def ward_segment(G, feature, stop=-1, qmax=1, verbose=False):
     """
     Agglomerative function based on a topology-defining graph
     and a feature matrix.
 
     Parameters
     ----------
-    G the input graph (a topological graph essentially)
-    feature array of shape (G.V,dim_feature)
-            some vectorial information related to the graph vertices
-    stop = -1: the stopping crterion
-         if stop==-1, then no stopping criterion is used
-    qmax=1: the maximum number of desired clusters
-            (in the limit of the stopping criterion)
+    G : graph object
+        the input graph (a topological graph essentially)
+    feature : array of shape (G.V,dim_feature)
+        some vectorial information related to the graph vertices
+    stop : int or float, optional
+        the stopping crterion.  if stop==-1, then no stopping criterion is used
+    qmax : int, optional
+        the maximum number of desired clusters (in the limit of the stopping
+        criterion)
+    verbose : bool, optional
+        If True, print diagnostic information
 
     Returns
     -------
@@ -973,11 +825,12 @@ def ward_segment(G, feature, stop=-1, qmax=1, verbose=0):
     cost: array of shape (G.V - 1)
           the cost of each merge step during the clustering procedure
 
-    NOTE
-    ----
+    Notes
+    -----
     A euclidean distance is used in the feature space
-    caveat : when the number of cc in G (nbcc)
-           is greter than qmax, u contains nbcc values, not qmax !
+
+    Caveat : when the number of cc in G (nbcc) is greter than qmax, u contains
+    nbcc values, not qmax !
     """
     # basic check
     if feature.ndim == 1:
@@ -992,7 +845,7 @@ def ward_segment(G, feature, stop=-1, qmax=1, verbose=0):
     if qmax == -1:
         qmax = n - 1
     if stop == -1:
-        stop = np.infty
+        stop = np.inf
     qmax = int(np.minimum(qmax, n - 1))
 
     t = ward(G, feature, verbose)
@@ -1013,25 +866,29 @@ def ward_segment(G, feature, stop=-1, qmax=1, verbose=0):
     return u, cost
 
 
-def ward(G, feature, verbose=0):
+def ward(G, feature, verbose=False):
     """
     Agglomerative function based on a topology-defining graph
     and a feature matrix.
 
-    Parameters:
-    ------------
-    G the input graph (a topological graph essentially)
-    feature array of shape (G.V,dim_feature)
-            vectorial information related to the graph vertices
+    Parameters
+    ----------
+    G : graph
+        the input graph (a topological graph essentially)
+    feature : array of shape (G.V,dim_feature)
+        vectorial information related to the graph vertices
+    verbose : bool, optional
+        If True, print diagnostic information
 
     Returns
     --------
-    t: a WeightedForest structure that represents the dendrogram
+    t : ``WeightedForest`` instance
+        structure that represents the dendrogram
 
-    Note
-    ----
-    When G has more than 1 connected component, t is no longer a tree.
-    This case is handled cleanly now
+    Notes
+    -----
+    When G has more than 1 connected component, t is no longer a tree.  This
+    case is handled cleanly now
     """
     # basic check
     if feature.ndim == 1:
@@ -1073,7 +930,7 @@ def ward(G, feature, verbose=0):
 
         # 2. remove the current edge
         K.edges[m] = - 1
-        K.weights[m] = np.infty
+        K.weights[m] = np.inf
         linc[i].remove(m)
         rinc[j].remove(m)
 
@@ -1081,7 +938,7 @@ def ward(G, feature, verbose=0):
         if np.sum(K.edges[ml, 1] == i) > 0:
             m = ml[np.flatnonzero(K.edges[ml, 1] == i)]
             K.edges[m] = -1
-            K.weights[m] = np.infty
+            K.weights[m] = np.inf
             linc[j].remove(m)
             rinc[i].remove(m)
 
@@ -1140,39 +997,3 @@ def _label(parent):
         labelled = _label_(root[j], parent, left, labelled)
 
     return labelled
-
-
-def _label2(parent, children):
-    # find the root
-    root = np.nonzero(parent == np.arange(np.size(parent)))[0]
-
-    # define left
-    left = np.zeros(np.size(parent))
-    for ch in children:
-        if np.size(ch) > 1:
-            left[ch[0]] = 1
-
-    left[root] = .5
-
-    # define labelled
-    labelled = - np.ones(np.size(parent))
-
-    # compute labelled
-    for j in range(np.size(root)):
-        labelled = _label_2(root[j], parent, left, labelled)
-
-    return labelled
-
-
-def _label_2(f, parent, left, labelled):
-    temp = np.nonzero(parent == f)
-    if np.size(temp) > 0:
-        i = temp[0][np.nonzero(left[temp[0]] == 1)]
-        j = temp[0][np.nonzero(left[temp[0]] == 0)]
-        labelled = _label_2(i, parent, left, labelled)
-        labelled[f] = labelled.max() + 1
-        for jj in j:
-            labelled = _label_2(jj, parent, left, labelled)
-    if labelled[f] < 0:
-        labelled[f] = labelled.max() + 1
-    return labelled
