@@ -3,9 +3,10 @@
 import numpy as np
 import scipy.linalg as spl
 
+from nibabel.affines import apply_affine
+
 from ...externals.transforms3d.quaternions import mat2quat, quat2axangle
 from .transform import Transform
-from ..utils.affines import apply_affine
 
 # Globals
 RADIUS = 100
@@ -85,11 +86,11 @@ def rotation_vec2mat(r):
     return R
 
 
-def matrix44(t, dtype=np.double):
+def to_matrix44(t, dtype=np.double):
     """
-    T = matrix44(t)
+    T = to_matrix44(t)
 
-    t is a vector of of affine transformation parameters with size at
+    t is a vector of affine transformation parameters with size at
     least 6.
 
     size < 6 ==> error
@@ -139,12 +140,65 @@ def inverse_affine(affine):
     return spl.inv(affine)
 
 
+def slices2aff(slices):
+    """ Return affine from start, step of sequence `slices` of slice objects
+
+    Parameters
+    ----------
+    slices : sequence of slice objects
+
+    Returns
+    -------
+    aff : ndarray
+        If ``N = len(slices)`` then affine is shape (N+1, N+1) with diagonal
+        given by the ``step`` attribute of the slice objects (where None
+        corresponds to 1), and the `:N` elements in the last column are given by
+        the ``start`` attribute of the slice objects
+
+    Examples
+    --------
+    >>> slices2aff([slice(None), slice(None)])
+    array([[ 1.,  0.,  0.],
+           [ 0.,  1.,  0.],
+           [ 0.,  0.,  1.]])
+    >>> slices2aff([slice(2, 3, 4), slice(3, 4, 5), slice(4, 5, 6)])
+    array([[ 4.,  0.,  0.,  2.],
+           [ 0.,  5.,  0.,  3.],
+           [ 0.,  0.,  6.,  4.],
+           [ 0.,  0.,  0.,  1.]])
+    """
+    starts = [s.start if not s.start is None else 0 for s in slices]
+    steps = [s.step if not s.step is None else 1 for s in slices]
+    aff = np.diag(steps + [1.])
+    aff[:-1, -1] = starts
+    return aff
+
+
 def subgrid_affine(affine, slices):
-    steps = map(lambda x: max(x, 1), [s.step for s in slices])
-    starts = map(lambda x: max(x, 0), [s.start for s in slices])
-    t = np.diag(np.concatenate((steps, [1]), 1))
-    t[0:3, 3] = starts
-    return np.dot(affine, t)
+    """ Return dot prodoct of `affine` and affine resulting from `slices`
+
+    Parameters
+    ----------
+    affine : array-like
+        Affine to apply on right of affine resulting from `slices`
+    slices : sequence of slice objects
+        Slices generating (N+1, N+1) affine from ``slices2aff``, where ``N =
+        len(slices)``
+
+    Returns
+    -------
+    aff : ndarray
+        result of ``np.dot(affine, slice_affine)`` where ``slice_affine`` is
+        affine resulting from ``slices2aff(slices)``.
+
+    Raises
+    ------
+    ValueError : if the ``slice_affine`` contains non-integer values
+    """
+    slices_aff = slices2aff(slices)
+    if not np.all(slices_aff == np.round(slices_aff)):
+           raise ValueError("Need integer slice start, step")
+    return np.dot(affine, slices_aff)
 
 
 class Affine(Transform):
@@ -155,7 +209,9 @@ class Affine(Transform):
         self._precond = preconditioner(radius)
         if array == None:
             self._vec12 = np.zeros(12)
-        elif array.size == 12:
+            return
+        array = np.array(array)
+        if array.size == 12:
             self._vec12 = array.ravel().copy()
         elif array.shape == (4, 4):
             self.from_matrix44(array)
@@ -246,7 +302,7 @@ class Affine(Transform):
     param = property(_get_param, _set_param)
 
     def as_affine(self, dtype='double'):
-        T = matrix44(self._vec12, dtype=dtype)
+        T = to_matrix44(self._vec12, dtype=dtype)
         if not self._direct:
             T[:3, :3] *= -1
         return T
