@@ -5,54 +5,53 @@
 #include <string.h>
 
 /* Useful marcos */ 
-
 #define ABS(a) ( (a) > 0.0 ? (a) : (-(a)) )
 #define FLOOR(a)((a)>0.0 ? (int)(a):(((int)(a)-a)!= 0.0 ? (int)(a)-1 : (int)(a)))  
 #define ROUND(a)(FLOOR(a+0.5))
 
-/* 
-   Compute left and right cubic spline neighbors. 
-   Use nx as auxiliary variable. 
+#ifdef _MSC_VER
+#define inline __inline
+#endif
 
+
+/*
+  Three different boundary conditions are implemented:
+      mode == 0 : 'zero'
+      mode == 1: 'nearest'
+      mode == 2: 'reflect'
+
+  Depending on the mode, the input coordinate x is mirrored so as to
+  fall within the image bounds [0..ddim] and a weight w is computed.
  */
-#define COMPUTE_NEIGHBORS(x, ddim, nx, px)		\
-  nx = (int)(x + ddim + 2);				\
-  if ((nx>=3) && (nx<=3*ddim)) {			\
-    px = nx - ddim;					\
-    nx = px - 3;					\
-  }							\
-  else							\
+#define APPLY_BOUNDARY_CONDITIONS(mode, x, w, ddim)		\
+  if (!_apply_boundary_conditions(mode, ddim, &x, &w))		\
     return 0.0;
 
-
-#define CUBIC_SPLINE_MIRROR(x, n, p)			\
-  ((x)<0.0 ? (-(x)) : ((x)>(n) ? ((p)-(x)) : (x)))
+#define COMPUTE_NEIGHBORS(x, ddim, nx, px)		\
+  if (!_mirror_grid_neighbors(x, ddim, &nx, &px))	\
+    return 0.0; 
 
 /* 
    The following marco forces numpy to consider a PyArrayIterObject
-   non-contiguous. Otherwise, coordinates won't be updated, apparently
-   for computation time reasons.  
+   non-contiguous. Otherwise, coordinates won't be updated - don't
+   know whether this is a bug or not. 
 */
 #define UPDATE_ITERATOR_COORDS(iter)		\
   iter->contiguous = 0;
 
 
-
 static void _cubic_spline_transform1d(double* res, double* src, unsigned int dim, 
 				      unsigned int res_stride, unsigned int src_stride); 
 static void _cubic_spline_transform(PyArrayObject* res, int axis, double* work); 
-static inline void _copy_double_buffer(double* res, double* src, unsigned int dim, unsigned int src_stride);
-
-static inline void _apply_affine_transform(double* Tx, 
-					   double* Ty, 
-					   double* Tz, 
+static inline void _copy_double_buffer(double* res, double* src, unsigned int dim,
+				       unsigned int src_stride);
+static inline int _mirrored_position(int x, unsigned int ddim);
+static inline int _apply_boundary_conditions(int mode, unsigned int ddim, 
+					     double* x, double* w);
+static inline int _mirror_grid_neighbors(double x, unsigned int ddim, int* nx, int* px);
+static inline void _apply_affine_transform(double* Tx, double* Ty, double* Tz, 
 					   const double* Tvox, 
-					   size_t x, 
-					   size_t y, 
-					   size_t z); 
-
-
-
+					   size_t x, size_t y, size_t z); 
 
 
 /* Numpy import */
@@ -97,8 +96,8 @@ static void _cubic_spline_transform1d(double* res, double* src, unsigned int dim
   int k; 
   double cp, cm, z1_k;
   double *buf_src, *buf_res;
-  const double z1 = -0.26794919243112; /* -2 + sqrt(3) */
-  const double cz1 = 0.28867513459481; /* z1/(z1^2-1) */
+  double z1 = -0.26794919243112; /* -2 + sqrt(3) */
+  double cz1 = 0.28867513459481; /* z1/(z1^2-1) */
 
   /* 
      Initial value for the causal recursion.
@@ -230,25 +229,12 @@ void cubic_spline_transform(PyArrayObject* res, const PyArrayObject* src)
 }
 
 
-/* 
-
-Assumes: -(dimX-1) <= x <= 2*(dimX-1) 
-and similarly for other coordinates. 
-
-Returns 0 otherwise. 
-
-*/
-
-double cubic_spline_sample1d (double x, const PyArrayObject* Coef) 
+double cubic_spline_sample1d (double x, const PyArrayObject* Coef, int mode) 
 {
 
-  unsigned int dim = PyArray_DIM(Coef, 0); 
+  unsigned int ddim = PyArray_DIM(Coef, 0) - 1; 
   unsigned int offset = PyArray_STRIDE(Coef, 0)/sizeof(double); 
   double *coef = PyArray_DATA(Coef); 
-
-  const unsigned int ddim = dim-1;
-  const unsigned int two_ddim = 2*ddim;
-
   double *buf;
   int nx, px, xx;
   double s;
@@ -256,8 +242,10 @@ double cubic_spline_sample1d (double x, const PyArrayObject* Coef)
   int posx[4];
   double *buf_bspx;
   int *buf_posx;
+  double w = 1; 
 
-  COMPUTE_NEIGHBORS(x, ddim, nx, px); 
+  APPLY_BOUNDARY_CONDITIONS(mode, x, w, ddim); 
+  COMPUTE_NEIGHBORS(x, ddim, nx, px);  
 
   /* Compute the B-spline values as well as the image positions 
      where to find the B-spline coefficients (including mirror conditions) */ 
@@ -265,7 +253,7 @@ double cubic_spline_sample1d (double x, const PyArrayObject* Coef)
   buf_posx = (int*)posx;
   for (xx = nx; xx <= px; xx ++, buf_bspx ++, buf_posx ++) {
     *buf_bspx = cubic_spline_basis(x-(double)xx);
-    *buf_posx = CUBIC_SPLINE_MIRROR(xx, ddim, two_ddim);
+    *buf_posx = _mirrored_position(xx, ddim);
   }
 
   /* Compute the interpolated value incrementally */
@@ -283,27 +271,21 @@ double cubic_spline_sample1d (double x, const PyArrayObject* Coef)
     
   }
     
-  return s;
-  
+  return w*s;
 }
 
 
 
 
-double cubic_spline_sample2d (double x, double y, const PyArrayObject* Coef)
+double cubic_spline_sample2d (double x, double y, const PyArrayObject* Coef,
+			      int mode_x, int mode_y)
 {
 
-  unsigned int dimX = PyArray_DIM(Coef, 0);
-  unsigned int dimY = PyArray_DIM(Coef, 1);
+  unsigned int ddimX = PyArray_DIM(Coef, 0) - 1;
+  unsigned int ddimY = PyArray_DIM(Coef, 1) - 1;
   unsigned int offX = PyArray_STRIDE(Coef, 0)/sizeof(double); 
   unsigned int offY = PyArray_STRIDE(Coef, 1)/sizeof(double); 
   double *coef = PyArray_DATA(Coef); 
-
-  const unsigned int ddimX = dimX-1;
-  const unsigned int ddimY = dimY-1;
-  const unsigned int two_ddimX = 2*ddimX;
-  const unsigned int two_ddimY = 2*ddimY;
-
   double *buf;
   int nx, ny, px, py, xx, yy;
   double s, aux;
@@ -312,8 +294,11 @@ double cubic_spline_sample2d (double x, double y, const PyArrayObject* Coef)
   double *buf_bspx, *buf_bspy;
   int *buf_posx, *buf_posy;
   int shfty;
+  double wx = 1, wy = 1; 
 
-  COMPUTE_NEIGHBORS(x, ddimX, nx, px);
+  APPLY_BOUNDARY_CONDITIONS(mode_x, x, wx, ddimX); 
+  COMPUTE_NEIGHBORS(x, ddimX, nx, px); 
+  APPLY_BOUNDARY_CONDITIONS(mode_y, y, wy, ddimY); 
   COMPUTE_NEIGHBORS(y, ddimY, ny, py);  
 
   /* Compute the B-spline values as well as the image positions 
@@ -322,14 +307,14 @@ double cubic_spline_sample2d (double x, double y, const PyArrayObject* Coef)
   buf_posx = (int*)posx;
   for (xx = nx; xx <= px; xx ++, buf_bspx ++, buf_posx ++) {
     *buf_bspx = cubic_spline_basis(x-(double)xx);
-    *buf_posx = CUBIC_SPLINE_MIRROR(xx, ddimX, two_ddimX);
+    *buf_posx = _mirrored_position(xx, ddimX);
   }
 
   buf_bspy = (double*)bspy;
   buf_posy = (int*)posy;
   for (yy = ny; yy <= py; yy ++, buf_bspy ++, buf_posy ++) {
     *buf_bspy = cubic_spline_basis(y-(double)yy);
-    *buf_posy = CUBIC_SPLINE_MIRROR(yy, ddimY, two_ddimY);
+    *buf_posy = _mirrored_position(yy, ddimY);
   }
 
 
@@ -359,28 +344,21 @@ double cubic_spline_sample2d (double x, double y, const PyArrayObject* Coef)
 
   }
   
-  return s;
+  return wx*wy*s;
 
 }
 
 
-double cubic_spline_sample3d (double x, double y, double z, const PyArrayObject* Coef)
+double cubic_spline_sample3d (double x, double y, double z, const PyArrayObject* Coef,
+			      int mode_x, int mode_y, int mode_z)
 {
-  unsigned int dimX = PyArray_DIM(Coef, 0);
-  unsigned int dimY = PyArray_DIM(Coef, 1);
-  unsigned int dimZ = PyArray_DIM(Coef, 2);
+  unsigned int ddimX = PyArray_DIM(Coef, 0) - 1;
+  unsigned int ddimY = PyArray_DIM(Coef, 1) - 1;
+  unsigned int ddimZ = PyArray_DIM(Coef, 2) - 1;
   unsigned int offX = PyArray_STRIDE(Coef, 0)/sizeof(double); 
   unsigned int offY = PyArray_STRIDE(Coef, 1)/sizeof(double); 
   unsigned int offZ = PyArray_STRIDE(Coef, 2)/sizeof(double); 
   double *coef = PyArray_DATA(Coef); 
-
-  const unsigned int ddimX = dimX-1;
-  const unsigned int ddimY = dimY-1;
-  const unsigned int ddimZ = dimZ-1;
-  const unsigned int two_ddimX = 2*ddimX;
-  const unsigned int two_ddimY = 2*ddimY;
-  const unsigned int two_ddimZ = 2*ddimZ;
-
   double *buf;
   int nx, ny, nz, px, py, pz;
   int xx, yy, zz;
@@ -390,10 +368,14 @@ double cubic_spline_sample3d (double x, double y, double z, const PyArrayObject*
   double *buf_bspx, *buf_bspy, *buf_bspz;
   int *buf_posx, *buf_posy, *buf_posz;
   int shftyz, shftz;
-  
-  COMPUTE_NEIGHBORS(x, ddimX, nx, px);
+  double wx = 1, wy = 1, wz = 1; 
+
+  APPLY_BOUNDARY_CONDITIONS(mode_x, x, wx, ddimX); 
+  COMPUTE_NEIGHBORS(x, ddimX, nx, px); 
+  APPLY_BOUNDARY_CONDITIONS(mode_y, y, wy, ddimY); 
   COMPUTE_NEIGHBORS(y, ddimY, ny, py);  
-  COMPUTE_NEIGHBORS(z, ddimZ, nz, pz);
+  APPLY_BOUNDARY_CONDITIONS(mode_z, z, wz, ddimZ); 
+  COMPUTE_NEIGHBORS(z, ddimZ, nz, pz);  
 
   /* Compute the B-spline values as well as the image positions 
      where to find the B-spline coefficients (including mirror conditions) */ 
@@ -401,21 +383,21 @@ double cubic_spline_sample3d (double x, double y, double z, const PyArrayObject*
   buf_posx = (int*)posx;
   for (xx = nx; xx <= px; xx ++, buf_bspx ++, buf_posx ++) {
     *buf_bspx = cubic_spline_basis(x-(double)xx);
-    *buf_posx = CUBIC_SPLINE_MIRROR(xx, ddimX, two_ddimX);
+    *buf_posx = _mirrored_position(xx, ddimX);
   }
 
   buf_bspy = (double*)bspy;
   buf_posy = (int*)posy;
   for (yy = ny; yy <= py; yy ++, buf_bspy ++, buf_posy ++) {
     *buf_bspy = cubic_spline_basis(y-(double)yy);
-    *buf_posy = CUBIC_SPLINE_MIRROR(yy, ddimY, two_ddimY);
+    *buf_posy = _mirrored_position(yy, ddimY);
   }
 
   buf_bspz = (double*)bspz;
   buf_posz = (int*)posz;
   for (zz = nz; zz <= pz; zz ++, buf_bspz ++, buf_posz ++) {
     *buf_bspz = cubic_spline_basis(z-(double)zz);
-    *buf_posz = CUBIC_SPLINE_MIRROR(zz, ddimZ, two_ddimZ);
+    *buf_posz = _mirrored_position(zz, ddimZ);
   }
 
   /* Compute the interpolated value incrementally */
@@ -454,33 +436,24 @@ double cubic_spline_sample3d (double x, double y, double z, const PyArrayObject*
   } /* end loop on z */
 
 
-  return s;
+  return wx*wy*wz*s;
 
 }
 
 
 
-double cubic_spline_sample4d (double x, double y, double z, double t, const PyArrayObject* Coef)
+double cubic_spline_sample4d (double x, double y, double z, double t, const PyArrayObject* Coef,
+			      int mode_x, int mode_y, int mode_z, int mode_t)
 {
-  unsigned int dimX = PyArray_DIM(Coef, 0);
-  unsigned int dimY = PyArray_DIM(Coef, 1);
-  unsigned int dimZ = PyArray_DIM(Coef, 2);
-  unsigned int dimT = PyArray_DIM(Coef, 3);
+  unsigned int ddimX = PyArray_DIM(Coef, 0) - 1;
+  unsigned int ddimY = PyArray_DIM(Coef, 1) - 1;
+  unsigned int ddimZ = PyArray_DIM(Coef, 2) - 1;
+  unsigned int ddimT = PyArray_DIM(Coef, 3) - 1;
   unsigned int offX = PyArray_STRIDE(Coef, 0)/sizeof(double); 
   unsigned int offY = PyArray_STRIDE(Coef, 1)/sizeof(double); 
   unsigned int offZ = PyArray_STRIDE(Coef, 2)/sizeof(double); 
   unsigned int offT = PyArray_STRIDE(Coef, 3)/sizeof(double); 
   double *coef = PyArray_DATA(Coef); 
-
-  const unsigned int ddimX = dimX-1;
-  const unsigned int ddimY = dimY-1;
-  const unsigned int ddimZ = dimZ-1;
-  const unsigned int ddimT = dimT-1;
-  const unsigned int two_ddimX = 2*ddimX;
-  const unsigned int two_ddimY = 2*ddimY;
-  const unsigned int two_ddimZ = 2*ddimZ;
-  const unsigned int two_ddimT = 2*ddimT;
-
   double *buf;
   int nx, ny, nz, nt, px, py, pz, pt;
   int xx, yy, zz, tt;
@@ -490,11 +463,16 @@ double cubic_spline_sample4d (double x, double y, double z, double t, const PyAr
   double *buf_bspx, *buf_bspy, *buf_bspz, *buf_bspt;
   int *buf_posx, *buf_posy, *buf_posz, *buf_post;
   int shftyzt, shftzt, shftt;
-  
-  COMPUTE_NEIGHBORS(x, ddimX, nx, px);
+  double wx = 1, wy = 1, wz = 1, wt = 1; 
+
+  APPLY_BOUNDARY_CONDITIONS(mode_x, x, wx, ddimX); 
+  COMPUTE_NEIGHBORS(x, ddimX, nx, px); 
+  APPLY_BOUNDARY_CONDITIONS(mode_y, y, wy, ddimY); 
   COMPUTE_NEIGHBORS(y, ddimY, ny, py);  
-  COMPUTE_NEIGHBORS(z, ddimZ, nz, pz);
-  COMPUTE_NEIGHBORS(t, ddimT, nt, pt);
+  APPLY_BOUNDARY_CONDITIONS(mode_z, z, wz, ddimZ); 
+  COMPUTE_NEIGHBORS(z, ddimZ, nz, pz);    
+  APPLY_BOUNDARY_CONDITIONS(mode_t, t, wt, ddimT); 
+  COMPUTE_NEIGHBORS(t, ddimT, nt, pt); 
 
   /* Compute the B-spline values as well as the image positions 
      where to find the B-spline coefficients (including mirror conditions) */ 
@@ -502,28 +480,28 @@ double cubic_spline_sample4d (double x, double y, double z, double t, const PyAr
   buf_posx = (int*)posx;
   for (xx = nx; xx <= px; xx ++, buf_bspx ++, buf_posx ++) {
     *buf_bspx = cubic_spline_basis(x-(double)xx);
-    *buf_posx = CUBIC_SPLINE_MIRROR(xx, ddimX, two_ddimX);
+    *buf_posx = _mirrored_position(xx, ddimX);
   }
 
   buf_bspy = (double*)bspy;
   buf_posy = (int*)posy;
   for (yy = ny; yy <= py; yy ++, buf_bspy ++, buf_posy ++) {
     *buf_bspy = cubic_spline_basis(y-(double)yy);
-    *buf_posy = CUBIC_SPLINE_MIRROR(yy, ddimY, two_ddimY);
+    *buf_posy = _mirrored_position(yy, ddimY);
   }
 
   buf_bspz = (double*)bspz;
   buf_posz = (int*)posz;
   for (zz = nz; zz <= pz; zz ++, buf_bspz ++, buf_posz ++) {
     *buf_bspz = cubic_spline_basis(z-(double)zz);
-    *buf_posz = CUBIC_SPLINE_MIRROR(zz, ddimZ, two_ddimZ);
+    *buf_posz = _mirrored_position(zz, ddimZ);
   }
 
   buf_bspt = (double*)bspt;
   buf_post = (int*)post;
   for (tt = nt; tt <= pt; tt ++, buf_bspt ++, buf_post ++) {
     *buf_bspt = cubic_spline_basis(t-(double)tt);
-    *buf_post = CUBIC_SPLINE_MIRROR(tt, ddimT, two_ddimT);
+    *buf_post = _mirrored_position(tt, ddimT);
   }
   
 
@@ -572,7 +550,7 @@ double cubic_spline_sample4d (double x, double y, double z, double t, const PyAr
     
   } /* end loop on t */
   
-  return s;
+  return wx*wy*wz*wt*s;
 
 }
 
@@ -581,10 +559,9 @@ double cubic_spline_sample4d (double x, double y, double z, double t, const PyAr
    Resample a 3d image submitted to an affine transformation.
    Tvox is the voxel transformation from the image to the destination grid.  
 */
-void cubic_spline_resample3d(PyArrayObject* im_resampled, 
-			     const PyArrayObject* im, 
-			     const double* Tvox, 
-			     int cast_integer)
+void cubic_spline_resample3d(PyArrayObject* im_resampled, const PyArrayObject* im,   
+			     const double* Tvox, int cast_integer, 
+			     int mode_x, int mode_y, int mode_z)
 {
   double i1;
   PyObject* py_i1;
@@ -594,7 +571,6 @@ void cubic_spline_resample3d(PyArrayObject* im_resampled,
   unsigned dimX = PyArray_DIM(im, 0);
   unsigned dimY = PyArray_DIM(im, 1);
   unsigned dimZ = PyArray_DIM(im, 2);
-  unsigned ddimX=dimX-1, ddimY=dimY-1, ddimZ=dimZ-1; 
   npy_intp dims[3] = {dimX, dimY, dimZ}; 
   double Tx, Ty, Tz;
 
@@ -611,16 +587,13 @@ void cubic_spline_resample3d(PyArrayObject* im_resampled,
     y = imIter->coordinates[1]; 
     z = imIter->coordinates[2]; 
     _apply_affine_transform(&Tx, &Ty, &Tz, Tvox, x, y, z); 
-
-    if ((Tx<0) || (Tx>ddimX) ||
-	(Ty<0) || (Ty>ddimY) ||
-	(Tz<0) || (Tz>ddimZ))
-      i1 = 0.0; 
-    else{ 
-      i1 = cubic_spline_sample3d(Tx, Ty, Tz, im_spline_coeff); 
-      if (cast_integer)
-	i1 = ROUND(i1); 
-    }
+    i1 = cubic_spline_sample3d(Tx, Ty, Tz, im_spline_coeff, mode_x, mode_y, mode_z); 
+    if (cast_integer) {
+      i1 = ROUND(i1);
+      if (cast_integer == 2)
+	if (i1 < 0)
+	  i1 = 0;
+     }
 
     /* Copy interpolated value into numpy array */
     py_i1 = PyFloat_FromDouble(i1); 
@@ -640,7 +613,8 @@ void cubic_spline_resample3d(PyArrayObject* im_resampled,
 }
 
 static inline void _apply_affine_transform(double* Tx, double* Ty, double* Tz, 
-					   const double* Tvox, size_t x, size_t y, size_t z)
+					   const double* Tvox,
+					   size_t x, size_t y, size_t z)
 {
   double* bufTvox = (double*)Tvox; 
 
@@ -659,3 +633,107 @@ static inline void _apply_affine_transform(double* Tx, double* Ty, double* Tz,
 
   return; 
 }
+
+
+/* 
+   Convert an input grid coordinate x into another grid coordinate
+   within [0, ddim], possibly using a reflection. This function
+   implicitely assumes that -ddim < x < 2*ddim
+ */
+static inline int _mirrored_position(int x, unsigned int ddim)
+{
+  if (x < 0)
+    return -x;
+  else if (x > ddim) 
+    return 2 * ddim - x;
+  else
+    return x;
+}
+
+/*
+Depending on the chosen mode, mirror the position and set the weight.
+*/
+static inline int _apply_boundary_conditions(int mode, unsigned int ddim, 
+					     double* x, double* w)
+{
+  int ok = 1; 
+  unsigned int dim = ddim + 1; 
+  int neg_ddim;
+  unsigned int two_ddim;
+
+  if (mode == 0) {
+    if (*x < -1)
+      ok = 0;
+    else if (*x < 0) {
+      *w = 1 + *x;
+      *x = 0;
+    }
+    else if (*x > dim)
+      ok = 0;
+    else if (*x > ddim) {
+      *w = dim - *x;
+      *x = ddim;
+    }
+  }
+  else if (mode == 1) {
+    if (*x < 0)
+      *x = 0;
+    else if (*x > ddim)
+      *x = ddim;
+  }
+  else{ /* mode==2 */
+    neg_ddim = -ddim; 
+    two_ddim = 2 * ddim; 
+    if ((*x < neg_ddim) || (*x > two_ddim))
+      ok = 0;
+  }
+  
+  return ok; 
+}
+
+/* 
+   Compute left and right cubic spline neighbors in the image grid
+   mirrored once on each side. Returns 0 if no neighbor can be found.
+*/
+static inline int _mirror_grid_neighbors(double x, unsigned int ddim, 
+					 int* nx, int* px)
+{
+  int ok = 0; 
+  *px = (int)(x+ddim+2);
+  if ((*px>=3) && (*px<=3*ddim)) {
+    ok = 1; 
+    *px = *px-ddim;
+    *nx = *px-3;
+  }		
+  return ok; 
+}
+
+
+
+static inline int _neighbors_zero_outside(double x, unsigned int ddim, 
+					  int* nx, int* px, 
+					  double* weight)
+{
+  int ok = 0, aux;
+  unsigned int dim = ddim+1;
+  *weight = 1; 
+
+  if ((x>-1) && (x<dim)) {
+      ok = 1; 
+      aux = (int)(x+2); 
+      *nx = aux-3; 
+      if (*nx<-1) { /* -1<x<0, nx==-2 */
+	*weight = 1-x;
+	*nx = -1;
+      }
+      *px = aux;
+      if (*px>dim) { /* ddim<=x<dim, px=dim+1 */ 
+	*weight = dim-x; 
+	*px = dim; 
+      }
+    }
+
+  return ok; 	
+}
+
+
