@@ -1,14 +1,16 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
 
-import gc
 import warnings
 import numpy as np
-from nibabel import io_orientation
 
-from ...core.image.affine_image import AffineImage
-from ..utils.affines import apply_affine
-from .image_utils import get_affine
+from nibabel.affines import apply_affine
+
+from ...fixes.nibabel import io_orientation
+
+from ...core.image.image_spaces import (make_xyz_image,
+                                        xyz_affine,
+                                        as_xyz_image)
 from .optimizer import configure_optimizer, use_derivatives
 from .affine import Rigid
 from ._registration import (_cspline_transform,
@@ -19,7 +21,7 @@ from ._registration import (_cspline_transform,
 # Module globals
 VERBOSE = True  # enables online print statements
 SLICE_ORDER = 'ascending'
-INTERLEAVED = False
+INTERLEAVED = None
 OPTIMIZER = 'ncg'
 XTOL = 1e-5
 FTOL = 1e-5
@@ -28,14 +30,21 @@ STEPSIZE = 1e-6
 SMALL = 1e-20
 MAXITER = 64
 MAXFUN = None
-LOOPS = 5, 1  # loops within each run
-BETWEEN_LOOPS = 5, 1  # loops used to realign different runs
-SPEEDUP = 5, 2
 BORDERS = 1, 1, 1
 REFSCAN = 0
 EXTRAPOLATE_SPACE = 'reflect'
 EXTRAPOLATE_TIME = 'reflect'
 
+LOOPS = 5  # loops within each run
+BETWEEN_LOOPS = 5  # loops used to realign different runs
+SPEEDUP = 5  # image sub-sampling factor for speeding up
+"""
+# How to tune those parameters for a multi-resolution implementation
+LOOPS = 5, 1
+BETWEEN_LOOPS = 5, 1
+SPEEDUP = 5, 2
+"""
+
 
 def interp_slice_order(Z, slice_order):
     Z = np.asarray(Z)
@@ -74,22 +83,13 @@ class Image4d(object):
 
     Parameters
     ----------
-    data : nd array or proxy (function that actually gets the array)
+      data : nd array or proxy (function that actually gets the array)
     """
     def __init__(self, data, affine, tr, tr_slices=None, start=0.0,
                  slice_order=SLICE_ORDER, interleaved=INTERLEAVED,
                  slice_info=None):
         """
         Configure fMRI acquisition time parameters.
-
-        tr  : inter-scan repetition time, i.e. the time elapsed
-              between two consecutive scans
-        tr_slices : inter-slice repetition time, same as tr for slices
-        start   : starting acquisition time respective to the implicit
-                  time origin
-        slice_order : string or array
-        slice_info : a tuple with slice axis as the first element and
-          direction as the second, for instance (2, 1)
         """
         self.affine = np.asarray(affine)
         self.tr = float(tr)
@@ -111,21 +111,32 @@ class Image4d(object):
 
         if isinstance(data, np.ndarray):
             self._data = data
+            self._shape = data.shape
             self._get_data = None
             self._init_timing_parameters()
         else:
             self._data = None
+            self._shape = None
             self._get_data = data
 
+    def _load_data(self):
+        self._data = self._get_data()
+        self._shape = self._data.shape
+        self._init_timing_parameters()
+
     def get_data(self):
         if self._data == None:
-            self._data = self._get_data()
-            self._init_timing_parameters()
+            self._load_data()
         return self._data
+    
+    def get_shape(self):
+        if self._shape == None:
+            self._load_data()
+        return self._shape
 
     def _init_timing_parameters(self):
         # Number of slices
-        nslices = self.get_data().shape[self.slice_axis]
+        nslices = self.get_shape()[self.slice_axis]
         self.nslices = nslices
         # Default slice repetition time (no silence)
         if self._tr_slices == None:
@@ -137,17 +148,20 @@ class Image4d(object):
             if not self.interleaved:
                 aux = range(nslices)
             else:
-                p = nslices / 2
-                aux = []
-                for i in range(p):
-                    aux.extend([i, p + i])
-                if nslices % 2:
-                    aux.append(nslices - 1)
+                aux = range(nslices)[0::2] + range(nslices)[1::2]
             if self._slice_order == 'descending':
                 aux.reverse()
             self.slice_order = np.array(aux)
         else:
-            self.slice_order = np.asarray(self.slice_order)
+            # Verify correctness of provided slice indexes
+            provided_slices = np.array(sorted(self._slice_order))
+            if np.any(provided_slices != np.arange(nslices)):
+                raise ValueError(
+                    "Incorrect slice indexes were provided. There are %d "
+                    "slices in the volume, indexes should start from 0 and "
+                    "list all slices. "
+                    "Provided slice_order: %s" % (nslices, self._slice_order))
+            self.slice_order = np.asarray(self._slice_order)
 
     def z_to_slice(self, z):
         """
@@ -172,8 +186,6 @@ class Image4d(object):
     def free_data(self):
         if not self._get_data == None:
             self._data = None
-        gc.enable()
-        gc.collect()
 
 
 class Realign4dAlgorithm(object):
@@ -195,7 +207,7 @@ class Realign4dAlgorithm(object):
                  maxfun=MAXFUN,
                  refscan=REFSCAN):
 
-        self.dims = im4d.get_data().shape
+        self.dims = im4d.get_shape()
         self.nscans = self.dims[3]
         self.xyz = make_grid(self.dims[0:3], subsampling, borders)
         masksize = self.xyz.shape[0]
@@ -421,7 +433,7 @@ class Realign4dAlgorithm(object):
         # multiscale pyramid. To avoid crashes, we insert a try/catch
         # instruction.
         try:
-            pc = fmin(f, self.transforms[t].param, *args, **kwargs)
+            pc = fmin(f, self.transforms[t].param, disp=VERBOSE, *args, **kwargs)
             self.set_transform(t, pc)
         except:
             warnings.warn('Minimization failed')
@@ -507,12 +519,12 @@ def single_run_realign4d(im4d,
       If a sequence, implement a multi-scale
 
     """
-    if not hasattr(loops, '__iter__'):
+    if not type(loops) in (list, tuple, np.array):
         loops = [loops]
     repeats = len(loops)
 
     def format_arg(x):
-        if not hasattr(x, '__iter__'):
+        if not type(x) in (list, tuple, np.array):
             x = [x for i in range(repeats)]
         else:
             if not len(x) == repeats:
@@ -535,7 +547,8 @@ def single_run_realign4d(im4d,
 
     for loops_, speedup_, optimizer_, xtol_, ftol_, gtol_,\
             stepsize_, maxiter_, maxfun_ in opt_params:
-        subsampling = adjust_subsampling(speedup_, im4d.get_data().shape[0:3])
+        subsampling = adjust_subsampling(speedup_, im4d.get_shape()[0:3])
+
         r = Realign4dAlgorithm(im4d,
                                transforms=transforms,
                                affine_class=affine_class,
@@ -595,7 +608,7 @@ def realign4d(runs,
     """
 
     # Single-session case
-    if not hasattr(runs, '__iter__'):
+    if not type(runs) in (list, tuple, np.array):
         runs = [runs]
     nruns = len(runs)
     if nruns == 1:
@@ -623,15 +636,14 @@ def realign4d(runs,
     # corrected run, and creating a fake time series with no temporal
     # smoothness
     ## FIXME: check that all runs have the same to-world transform
-    mean_img_shape = list(runs[0].get_data().shape[0:3]) + [nruns]
+    mean_img_shape = list(runs[0].get_shape()[0:3]) + [nruns]
     mean_img_data = np.zeros(mean_img_shape)
 
     for i in range(nruns):
         corr_run = resample4d(runs[i], transforms=transforms[i],
                               time_interp=time_interp)
         mean_img_data[..., i] = corr_run.mean(3)
-        gc.enable()
-        gc.collect()
+    del corr_run
 
     mean_img = Image4d(mean_img_data, affine=runs[0].affine,
                        tr=1.0, tr_slices=0.0)
@@ -665,12 +677,22 @@ class Realign4d(object):
     def _generic_init(self, images, affine_class,
                       slice_order, interleaved, tr, tr_slices,
                       start, time_interp, slice_info):
-        if not hasattr(images, '__iter__'):
+        if slice_order == None:
+            slice_order = SLICE_ORDER
+            if time_interp:
+                raise ValueError('Slice order is requested'
+                          + ' with time interpolation switched on')
+            time_interp = False
+        if tr == None:
+            raise ValueError('Repetition time cannot be None')
+        if not type(images) in (list, tuple, np.array):
             images = [images]
         self._runs = []
         self.affine_class = affine_class
         for im in images:
-            self._runs.append(Image4d(im.get_data, get_affine(im),
+            xyz_img = as_xyz_image(im)
+            self._runs.append(Image4d(xyz_img.get_data,
+                                      xyz_affine(xyz_img),
                                       tr=tr, tr_slices=tr_slices,
                                       start=start, slice_order=slice_order,
                                       interleaved=interleaved,
@@ -728,18 +750,81 @@ class Realign4d(object):
         if r == None:
             data = [resample4d(self._runs[r], transforms=transforms[r],
                                time_interp=self._time_interp) for r in runs]
-            return [AffineImage(data[r], self._runs[r].affine, 'scanner')\
-                        for r in runs]
+            return [make_xyz_image(data[r], self._runs[r].affine, 'scanner')
+                    for r in runs]
         else:
             data = resample4d(self._runs[r], transforms=transforms[r],
                               time_interp=self._time_interp)
-            return AffineImage(data, self._runs[r].affine, 'scanner')
+            return make_xyz_image(data, self._runs[r].affine, 'scanner')
 
 
 class FmriRealign4d(Realign4d):
 
-    def __init__(self, images, slice_order, interleaved,
-                 tr=None, tr_slices=None, start=0.0, time_interp=True,
+    def __init__(self, images, slice_order, interleaved=None,
+                 tr=1.0, tr_slices=None, start=0.0, time_interp=True,
                  affine_class=Rigid, slice_info=None):
+
+        """
+        Spatiotemporal realignment class for fMRI series.
+
+        Parameters
+        ----------
+        images : image or list of images
+          Single or multiple input 4d images representing one or
+          several fMRI runs.
+
+        tr : float
+          Inter-scan repetition time, i.e. the time elapsed between
+          two consecutive scans. The unit in which `tr` is given is
+          arbitrary although it needs to be consistent with the
+          `tr_slices` and `start` arguments.
+
+        tr_slices : float
+          Inter-slice repetition time, same as tr for slices. If None,
+          acquisition is assumed continuous and `tr_slices` is set to
+          `tr` divided by the number of slices.
+
+        start : float
+          Starting acquisition time respective to the implicit time
+          origin.
+
+        slice_order : str or array-like
+          If str, one of {'ascending', 'descending'}. If array-like,
+          then the order in which the slices were collected in
+          time. For instance, the following represents an ascending
+          contiguous sequence:
+
+          slice_order = [0, 1, 2, ...]
+
+        interleaved : bool
+          Deprecated.
+
+          Whether slice acquisition order is interleaved. Ignored if
+          `slice_order` is array-like.
+
+          If slice_order=='ascending' and interleaved==True, the
+          assumed slice order is:
+
+          [0, 2, 4, ..., 1, 3, 5, ...]
+
+          If slice_order=='descending' and interleaved==True, the
+          assumed slice order is:
+
+          [N-1, N-3, N-5, ..., N-2, N-4, N-6]
+
+          Given that there exist other types of interleaved
+          acquisitions depending on scanner settings and
+          manufacturers, it is strongly recommended to input the
+          slice_order as an array unless you are sure what you are
+          doing.
+
+        slice_info : None or tuple, optional
+          None, or a tuple with slice axis as the first element and
+          direction as the second, for instance (2, 1).  If None, then
+          guess the slice axis, and direction, as the closest to the z
+          axis, as estimated from the affine.
+        """
+        if not interleaved == None:
+            warnings.warn('interleaved keyword is deprecated. Please input explicit slice order instead.')
         self._generic_init(images, affine_class, slice_order, interleaved,
                            tr, tr_slices, start, time_interp, slice_info)
