@@ -8,7 +8,7 @@ def configuration(parent_package='', top_path=None):
     from numpy.distutils.misc_util import Configuration
 
     config = Configuration('registration', parent_package, top_path)
-    config.add_data_dir('tests')
+    config.add_subpackage('tests')
     config.add_include_dirs(config.name.replace('.', os.sep))
     config.add_extension(
         '_registration',
@@ -21,4 +21,4 @@ def configuration(parent_package='', top_path=None):
 
 
 if __name__ == '__main__':
-    print 'This is the wrong setup.py file to run'
+    print('This is the wrong setup.py file to run')
