@@ -1,16 +1,25 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
-"""
-This module contains several routines to get corrected p-values estimates, 
-based on the observation of data/p-values. It yields 3 main approaches:
-- benjamini-Hochberg fdr
-- a class that fits a gaussian model to the central
-part of an histogram, following schwartzman et al, 2009. This is
-typically necessary to estimate a fdr when one is not certain that the
-data behaves as a standard normal under H_0.
-- a model based on gaussian mixture modelling
+"""Routines to get corrected p-values estimates, based on the observations.
 
-Author : Bertrand Thirion, 2008-2011
+It implements 3 approaches:
+
+- Benjamini-Hochberg FDR: http://en.wikipedia.org/wiki/False_discovery_rate
+
+- a class that fits a Gaussian model to the central part of an
+  histogram, following [1]
+
+  [1] Schwartzman A, Dougherty RF, Lee J, Ghahremani D, Taylor
+  JE. Empirical null and false discovery rate analysis in
+  neuroimaging. Neuroimage. 2009 Jan 1;44(1):71-82.  PubMed PMID:
+  18547821. DOI: 10.1016/j.neuroimage.2008.04.182
+
+  This is typically necessary to estimate a FDR when one is not
+  certain that the data behaves as a standard normal under H_0.
+
+- a model based on Gaussian mixture modelling 'a la Oxford'
+
+Author : Bertrand Thirion, Yaroslav Halchenko, 2008-2012
 """
 
 import numpy as np
@@ -18,146 +27,135 @@ from numpy.linalg import pinv
 import scipy.stats as st
 
 
-def all_fdr_gaussian(x):
-    """Return the fdr of all values assuming a Gaussian distribution
+def check_p_values(p_values):
+    """Basic checks on the p_values array: values should be within [0,1]
+
+    Assures also that p_values are at least in 1d array.  None of the
+    checks is performed if p_values is None.
+
+    Parameters
+    ----------
+    p_values : array of shape (n)
+        The sample p-values
+
+    Returns
+    -------
+    p_values : array of shape (n)
+        The sample p-values
     """
-    pvals = st.norm.sf(np.squeeze(x))
-    return(FDR(pvals).all_fdr())
+    if p_values is None:
+        return None
+    # Take all elements unfolded and assure having at least 1d
+    p_values = np.atleast_1d(np.ravel(p_values))
+    if np.any(np.isnan(p_values)):
+        raise ValueError("%d values are NaN" % (sum(np.isnan(p_values))))
+    if p_values.min() < 0:
+        raise ValueError("Negative p-values. Min=%g" % (p_values.min(),))
+    if p_values.max() > 1:
+        raise ValueError("P-values greater than 1! Max=%g" % (
+                p_values.max(),))
+    return p_values
+
+
+def gaussian_fdr(x):
+    """Return the FDR associated with each value assuming a Gaussian distribution
+    """
+    return fdr(st.norm.sf(np.squeeze(x)))
+
 
 def gaussian_fdr_threshold(x, alpha=0.05):
-        """
-        Given an array x of normal variates, this function returns the
-        critical p-value associated with alpha.
-        x is explicitly assumed to be normal distributed under H_0
+    """Return FDR threshold given normal variates
 
-        Parameters
-        -----------
-        x: ndarray, the input data
-        alpha: float, optional, the desired significance
+    Given an array x of normal variates, this function returns the
+    critical p-value associated with alpha.
+    x is explicitly assumed to be normal distributed under H_0
 
-        Returns
-        -------
-        th: float,
-            The threshold in variate value
-        """
-        pvals = st.norm.sf(x)
-        pth = FDR(pvals).pth_from_pvals(pvals, alpha)
-        print pth, pvals.min(), alpha, x
-        return st.norm.isf(pth)
+    Parameters
+    -----------
+    x: ndarray
+        input data
+    alpha: float, optional
+        desired significance
 
-
-class FDR(object):
-    """ Basic class to handle false discovery rate computation
-    Members:
-    x the samples from which the fdr is derived, assumed to be a normal variate
-
-    The Benjamini-Horchberg procedure is used
+    Returns
+    -------
+    threshold : float
+        threshold, given as a Gaussian critical value
     """
+    pvals = st.norm.sf(x)
+    pth = fdr_threshold(pvals, alpha)
+    return st.norm.isf(pth)
 
-    def __init__(self, pv=None):
-        """
-        Parameters
-        ----------
-        pv: array of p-values
-        """
-        if pv is not None:
-            self.pv = self.check_pv(pv)
 
-    def all_fdr(self, pv=None, verbose=0):
-        """ Returns the fdr associated with each the values
+def fdr_threshold(p_values, alpha=0.05):
+    """Return FDR threshold given p values
 
-        Parameters
-        -----------
-        pv : ndarray of shape (n)
-            The samples p-value
+    Parameters
+    -----------
+    p_values : array of shape (n), optional
+        The samples p-value
+    alpha : float, optional
+        The desired FDR significance
 
-        Returns
-        --------
-        q : array of shape(n)
-            The corresponding fdrs
-        """
-        pv = self.check_pv(pv)
-        if pv == None:
-            pv = self.pv
-        if pv == None:
-            return None
-        n = np.size(pv)
-        isx = np.argsort(pv)
-        q = np.zeros(n)
-        for ip in range(n):
-            q[isx[ip]] = np.minimum(1, np.maximum(n * pv[isx[ip]] / (ip + 1),
-                                                  q[isx[ip]]))
-            if (ip < n - 1):
-                q[isx[ip + 1]] = q[isx[ip]]
+    Returns
+    -------
+    critical_p_value: float
+        The p value corresponding to the FDR alpha
+    """
+    p_values = check_p_values(p_values)
+    n_samples = np.size(p_values)
+    p_corr = alpha / n_samples
+    sp_values = np.sort(p_values)
+    critical_set = sp_values[
+        sp_values < p_corr * np.arange(1, n_samples + 1)]
+    if len(critical_set) > 0:
+        critical_p_value = critical_set.max()
+    else:
+        critical_p_value = p_corr
+    return critical_p_value
 
-        if verbose:
-            import matplotlib.pylab as mp
-            mp.figure()
-            mp.plot(pv, q, '.')
-        return q
 
-    def check_pv(self, pv):
-        """
-        Do some basic checks on the pv array: each value should be within [0,1]
+def fdr(p_values=None, verbose=0):
+    """Returns the FDR associated with each p value
 
-        Parameters
-        ----------
-        pv : array of shape (n)
-            The sample p-values
+    Parameters
+    -----------
+    p_values : ndarray of shape (n)
+        The samples p-value
 
-        Returns
-        --------
-        pv : array of shape (n)
-            The sample p-values
-        """
-        if pv is None:
-            return None
-        pv = np.squeeze(pv)
-        if pv.min() < 0:
-            print pv.min()
-            raise ValueError("Negative p-values")
-        if pv.max() > 1:
-            print pv.max()
-            raise ValueError("P-values greater than 1!")
-        if np.isscalar(pv):
-            pv = np.array([pv])
-        return pv
+    Returns
+    -------
+    q : array of shape(n)
+        The corresponding fdr values
+    """
+    p_values = check_p_values(p_values)
+    n_samples = p_values.size
+    order = p_values.argsort()
+    sp_values = p_values[order]
 
-    def pth_from_pvals(self, pv=None, alpha=0.05):
-        """ Returns the critical p-value associated with an FDR alpha
+    # compute q while in ascending order
+    q = np.minimum(1, n_samples * sp_values / np.arange(1, n_samples + 1))
+    for i in range(n_samples - 1, 0, - 1):
+        q[i - 1] = min(q[i], q[i - 1])
+
+    # reorder the results
+    inverse_order = np.arange(n_samples)
+    inverse_order[order] = np.arange(n_samples)
+    q = q[inverse_order]
+
+    if verbose:
+        import matplotlib.pylab as mp
+        mp.figure()
+        mp.xlabel('Input p-value')
+        mp.plot(p_values, q, '.')
+        mp.ylabel('Associated fdr')
+    return q
 
-        Parameters
-        -----------
-        pv : array of shape (n), optional
-            The samples p-value
-        alpha : float, optional
-            The desired FDR significance
-        
-        Returns
-        -------
-        pth: float
-            The p value corresponding to the FDR alpha
-        """
-        pv = self.check_pv(pv)
-        if pv == None:
-            pv = self.pv
-        if pv == None:
-            return None
-        npv = np.size(pv)
-        pcorr = alpha / npv
-        spv = np.sort(pv)
-        ip = 0
-        pth = 0.
-        while (spv[ip] < pcorr * (ip + 1)) & (ip < npv):
-            pth = spv[ip]
-            ip += 1
-        return pth
-    
 
 class NormalEmpiricalNull(object):
     """Class to compute the empirical null normal fit to the data.
 
-    The data which is used to estimate the FDR, assuming a gaussian null
+    The data which is used to estimate the FDR, assuming a Gaussian null
     from Schwartzmann et al., NeuroImage 44 (2009) 71--82
     """
 
@@ -169,32 +167,33 @@ class NormalEmpiricalNull(object):
         x : 1D ndarray
             The data used to estimate the empirical null.
         """
-        x = np.reshape(x, ( - 1))
+        x = np.reshape(x, (- 1))
         self.x = np.sort(x)
         self.n = np.size(x)
         self.learned = 0
 
     def learn(self, left=0.2, right=0.8):
         """
-        Estimate the proportion, mean and variance of a gaussian distribution
+        Estimate the proportion, mean and variance of a Gaussian distribution
         for a fraction of the data
 
         Parameters
         ----------
         left: float, optional
-              Left cut parameter to prevent fitting non-gaussian data
+            Left cut parameter to prevent fitting non-gaussian data
         right: float, optional
-               Right cut parameter to prevent fitting non-gaussian data
+            Right cut parameter to prevent fitting non-gaussian data
 
-        Note
-        ----
+        Notes
+        -----
         This method stores the following attributes:
-        mu = mu
-        p0 = min(1, np.exp(lp0))
-        sqsigma: standard deviation of the estimated normal
-                 distribution
-        sigma: np.sqrt(sqsigma) : variance of the estimated
-               normal distribution
+
+        * mu = mu
+        * p0 = min(1, np.exp(lp0))
+        * sqsigma: standard deviation of the estimated normal
+          distribution
+        * sigma: np.sqrt(sqsigma) : variance of the estimated
+          normal distribution
         """
         # take a central subsample of x
         x = self.x[int(self.n * left): int(self.n * right)]
@@ -228,7 +227,7 @@ class NormalEmpiricalNull(object):
 
     def fdrcurve(self):
         """
-        Returns the fdr associated with any point of self.x
+        Returns the FDR associated with any point of self.x
         """
         import scipy.stats as st
         if self.learned == 0:
@@ -245,7 +244,7 @@ class NormalEmpiricalNull(object):
 
     def threshold(self, alpha=0.05, verbose=0):
         """
-        Compute the threshold correponding to an alpha-level fdr for x
+        Compute the threshold corresponding to an alpha-level FDR for x
 
         Parameters
         -----------
@@ -254,10 +253,10 @@ class NormalEmpiricalNull(object):
         verbose : boolean, optional
             the verbosity level, if True a plot is generated.
 
-        Results
-        --------
+        Returns
+        -------
         theta: float
-            the critical value associated with the provided fdr
+            the critical value associated with the provided FDR
         """
         efp = self.fdrcurve()
 
@@ -265,24 +264,24 @@ class NormalEmpiricalNull(object):
             self.plot(efp, alpha)
 
         if efp[-1] > alpha:
-            print "the maximal value is %f , the corresponding fdr is %f " \
+            print "the maximal value is %f , the corresponding FDR is %f " \
                     % (self.x[ - 1], efp[ - 1])
-            return np.infty
+            return np.inf
         j = np.argmin(efp[:: - 1] < alpha) + 1
         return 0.5 * (self.x[ - j] + self.x[ - j + 1])
 
     def uncorrected_threshold(self, alpha=0.001, verbose=0):
-        """ Compute the threshold correponding to a specificity alpha for x
+        """Compute the threshold corresponding to a specificity alpha for x
 
         Parameters
         -----------
         alpha : float, optional
-            the chosen false discovery rate threshold.
+            the chosen false discovery rate (FDR) threshold.
         verbose : boolean, optional
             the verbosity level, if True a plot is generated.
 
-        Results
-        --------
+        Returns
+        -------
         theta: float
             the critical value associated with the provided p-value
         """
@@ -296,16 +295,16 @@ class NormalEmpiricalNull(object):
         return threshold
 
     def fdr(self, theta):
-        """Given a threshold theta, find the estimated fdr
+        """Given a threshold theta, find the estimated FDR
 
-        Parameter
-        ---------
-        theta: float or array of shape (n_samples)
-               values to test
+        Parameters
+        ----------
+        theta : float or array of shape (n_samples)
+            values to test
 
         Returns
         -------
-        afp: value of array of shape(n)
+        afp : value of array of shape(n)
         """
         from scipy.stats import norm
         self.fdrcurve()
@@ -331,16 +330,16 @@ class NormalEmpiricalNull(object):
         return efp
 
     def plot(self, efp=None, alpha=0.05, bar=1, mpaxes=None):
-        """plot the  histogram of x
+        """Plot the  histogram of x
 
         Parameters
         ------------
         efp : float, optional
-            The empirical fdr (corresponding to x)
-            if efp==None, the false positive rate threshod plot is not
+            The empirical FDR (corresponding to x)
+            if efp==None, the false positive rate threshold plot is not
             drawn.
         alpha : float, optional
-            The chosen fdr threshold
+            The chosen FDR threshold
         bar=1 : bool, optional
         mpaxes=None: if not None, handle to an axes where the fig
         will be drawn. Avoids creating unnecessarily new figures
@@ -364,14 +363,14 @@ class NormalEmpiricalNull(object):
         else:
             ax = mpaxes
         if bar:
-            # We need to cut ledge to len(hist) to accomodate for pre and
+            # We need to cut ledge to len(hist) to accommodate for pre and
             # post numpy 1.3 hist semantic change.
             ax.bar(ledge[:len(hist)], hist, step)
         else:
             ax.plot(medge[:len(hist)], hist, linewidth=2)
         ax.plot(medge, g, 'r', linewidth=2)
         ax.set_title('Robust fit of the histogram', fontsize=12)
-        l = ax.legend(('empiricall null', 'data'), loc=0)
+        l = ax.legend(('empirical null', 'data'), loc=0)
         for t in l.get_texts():
             t.set_fontsize(12)
         ax.set_xticklabels(ax.get_xticks(), fontsize=12)
@@ -384,35 +383,41 @@ class NormalEmpiricalNull(object):
 def three_classes_GMM_fit(x, test=None, alpha=0.01, prior_strength=100,
                           verbose=0, fixed_scale=False, mpaxes=None, bias=0,
                           theta=0, return_estimator=False):
-    """ Fit the data with a 3-classes Gaussian Mixture Model,
-    i.e. computing some probability that the voxels of a certain map
+    """Fit the data with a 3-classes Gaussian Mixture Model,
+    i.e. compute some probability that the voxels of a certain map
     are in class disactivated, null or active
 
     Parameters
     ----------
-    x array of shape (nvox,1): the map to be analysed
-    test=None array of shape(nbitems,1):
+    x: array of shape (nvox,1)
+      The map to be analysed
+    test: array of shape(nbitems,1), optional
       the test values for which the p-value needs to be computed
-      by default, test=x
-    alpha = 0.01 the prior weights of the positive and negative classes
-    prior_strength = 100 the confidence on the prior
-                   (should be compared to size(x))
-    verbose=0 : verbosity mode
-    fixed_scale = False, boolean, variance parameterization
-                if True, the variance is locked to 1
-                otherwise, it is estimated from the data
-    mpaxes=None: axes handle used to plot the figure in verbose mode
-                 if None, new axes are created
-    bias = 0: allows a recaling of the posterior probability
-         that takes into account the thershold theta. Not rigorous.
-    theta = 0 the threshold used to correct the posterior p-values
-          when bias=1; normally, it is such that test>theta
-          note that if theta = -np.infty, the method has a standard behaviour
+      by default (if None), test=x
+    alpha: float, optional
+      the prior weights of the positive and negative classes
+    prior_strength: float, optional
+      the confidence on the prior (should be compared to size(x))
+    verbose: int
+      verbosity mode
+    fixed_scale: bool, optional
+      boolean, variance parameterization. if True, the variance is locked to 1
+      otherwise, it is estimated from the data
+    mpaxes:
+      axes handle used to plot the figure in verbose mode
+      if None, new axes are created
+    bias:  bool
+      allows a rescaling of the posterior probability
+      that takes into account the threshold theta. Not rigorous.
+    theta: float
+      the threshold used to correct the posterior p-values
+      when bias=1; normally, it is such that test>theta
+      note that if theta = -np.inf, the method has a standard behaviour
     return_estimator: boolean, optional
-            If return_estimator is true, the estimator object is
-            returned.
+      If return_estimator is true, the estimator object is
+      returned.
 
-    Results
+    Returns
     -------
     bfp : array of shape (nbitems,3):
         the posterior probability of each test item belonging to each component
@@ -421,12 +426,13 @@ def three_classes_GMM_fit(x, test=None, alpha=0.01, prior_strength=100,
     estimator : nipy.labs.clustering.GMM object
         The estimator object, returned only if return_estimator is true.
 
-    Note
-    ----
-    Our convention is that
-    - class 1 represents the negative class
-    - class 2 represenst the null class
-    - class 3 represents the positsive class
+    Notes
+    -----
+    Our convention is that:
+
+    * class 1 represents the negative class
+    * class 2 represents the null class
+    * class 3 represents the positive class
     """
     from ..clustering.bgmm import VBGMM
     from ..clustering.gmm import GridDescriptor
@@ -488,11 +494,11 @@ def three_classes_GMM_fit(x, test=None, alpha=0.01, prior_strength=100,
         return bfp, BayesianGMM
 
 
-def Gamma_Gaussian_fit(x, test=None, verbose=0, mpaxes=None,
+def gamma_gaussian_fit(x, test=None, verbose=0, mpaxes=None,
                        bias=1, gaussian_mix=0, return_estimator=False):
     """
     Computing some prior probabilities that the voxels of a certain map
-    are in class disactivated, null or active uning a gamma-Gaussian mixture
+    are in class disactivated, null or active using a gamma-Gaussian mixture
 
     Parameters
     ------------
@@ -508,19 +514,19 @@ def Gamma_Gaussian_fit(x, test=None, verbose=0, mpaxes=None,
         axes handle used to plot the figure in verbose mode
         if None, new axes are created
     bias: float, optional
-            lower bound on the gaussian variance (to avoid shrinkage)
+        lower bound on the Gaussian variance (to avoid shrinkage)
     gaussian_mix: float, optional
-            if nonzero, lower bound on the gaussian mixing weight
-            (to avoid shrinkage)
+        if nonzero, lower bound on the Gaussian mixing weight
+        (to avoid shrinkage)
     return_estimator: boolean, optional
-            If return_estimator is true, the estimator object is
-            returned.
+        if return_estimator is true, the estimator object is
+        returned.
 
     Returns
     -------
     bfp: array of shape (nbitems,3)
-            The probability of each component in the mixture model for each
-            test value
+        The probability of each component in the mixture model for each
+        test value
     estimator: nipy.labs.clustering.ggmixture.GGGM object
         The estimator object, returned only if return_estimator is true.
     """
@@ -545,7 +551,7 @@ def Gamma_Gaussian_fit(x, test=None, verbose=0, mpaxes=None,
 
 
 def smoothed_histogram_from_samples(x, bins=None, nbins=256, normalized=False):
-    """ Returns the smooth histogram corresponding to the  density
+    """Returns the smooth histogram corresponding to the  density
     underlying the samples in x
 
     Parameters
@@ -555,16 +561,16 @@ def smoothed_histogram_from_samples(x, bins=None, nbins=256, normalized=False):
     bins: array of shape(nbins+1), optional,
        the bins location
     nbins: int, optional,
-           the number of bins of the resulting histogram
-    Normalized: bool, optional
-                if True, the result is returned as a density value
+       the number of bins of the resulting histogram
+    normalized: bool, optional
+       if True, the result is returned as a density value
 
     Returns
     -------
     h: array of shape (nbins)
        the histogram
     bins: array of shape(nbins+1),
-          the bins location
+       the bins location
     """
     from scipy.ndimage import gaussian_filter1d
 
