@@ -1,6 +1,6 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
-from .glm import *
+from .glm import models, contrast, ols, load
 
 from nipy.testing import Tester
 test = Tester().test
