@@ -14,22 +14,22 @@ from nibabel import load, nifti1, save
 from nibabel.loadsave import read_img_data
 
 
-################################################################################
+###############################################################################
 # Operating on connect component
-################################################################################
+###############################################################################
 
 def largest_cc(mask):
     """ Return the largest connected component of a 3D mask array.
 
-        Parameters
-        -----------
-        mask: 3D boolean array
-            3D array indicating a mask.
-        
-        Returns
-        --------
-        mask: 3D boolean array 
-            3D array indicating a mask, with only one connected component.    
+    Parameters
+    -----------
+    mask: 3D boolean array
+          3D array indicating a mask.
+
+    Returns
+    --------
+    mask: 3D boolean array
+          3D array indicating a mask, with only one connected component.
     """
     # We use asarray to be able to work with masked arrays.
     mask = np.asarray(mask)
@@ -41,7 +41,7 @@ def largest_cc(mask):
     label_count = np.bincount(labels.ravel())
     # discard 0 the 0 label
     label_count[0] = 0
-    return labels ==  label_count.argmax() 
+    return labels == label_count.argmax()
 
 
 def threshold_connect_components(map, threshold, copy=True):
@@ -49,14 +49,19 @@ def threshold_connect_components(map, threshold, copy=True):
         connect components with number of voxels smaller than the
         threshold and set them to 0.
 
-        Parameters
-        ----------
-        map: ndarray
-            The map to segment
-        threshold:
-            The minimum number of voxels to keep a cluster.
-        copy: bool, optional
-            If copy is false, the input array is modified inplace
+    Parameters
+    ----------
+    map: ndarray,
+         The spatial map to segment
+    threshold: scalar,
+               The minimum number of voxels to keep a cluster.
+    copy: bool, optional
+          If copy is false, the input array is modified inplace
+
+    Returns
+    -------
+    map: ndarray,
+         the map with connected components removed
     """
     labels, _ = ndimage.label(map)
     weights = np.bincount(labels.ravel())
@@ -70,12 +75,13 @@ def threshold_connect_components(map, threshold, copy=True):
     return map
 
 
-################################################################################
+###############################################################################
 # Utilities to calculate masks
-################################################################################
+###############################################################################
 
-def compute_mask_files(input_filename, output_filename=None, 
-                        return_mean=False, m=0.2, M=0.9, cc=1):
+def compute_mask_files(input_filename, output_filename=None,
+                        return_mean=False, m=0.2, M=0.9, cc=1,
+                        exclude_zeros=False, opening=2):
     """
     Compute a mask file from fMRI nifti file(s)
 
@@ -85,7 +91,7 @@ def compute_mask_files(input_filename, output_filename=None,
     m and M of the total image histogram.
 
     In case of failure, it is usually advisable to increase m.
-   
+
     Parameters
     ----------
     input_filename : string
@@ -93,7 +99,7 @@ def compute_mask_files(input_filename, output_filename=None,
     output_filename : string or None, optional
         path to save the output nifti image (if not None).
     return_mean : boolean, optional
-        if True, and output_filename is None, return the mean image also, as 
+        if True, and output_filename is None, return the mean image also, as
         a 3D array (2nd return argument).
     m : float, optional
         lower fraction of the histogram to be discarded.
@@ -101,18 +107,24 @@ def compute_mask_files(input_filename, output_filename=None,
         upper fraction of the histogram to be discarded.
     cc: boolean, optional
         if cc is True, only the largest connect component is kept.
+    exclude_zeros: boolean, optional
+        Consider zeros as missing values for the computation of the
+        threshold. This option is useful if the images have been
+        resliced with a large padding of zeros.
+    opening: int, optional
+        Size of the morphological opening performed as post-processing
 
     Returns
     -------
-    mask : 3D boolean array 
+    mask : 3D boolean array
         The brain mask
     mean_image : 3d ndarray, optional
         The main of all the images used to estimate the mask. Only
         provided if `return_mean` is True.
     """
     if isinstance(input_filename, basestring):
-        # One single filename
-        nim = load(input_filename)
+        # One single filename or image
+        nim = load(input_filename)  # load the image from the path
         vol_arr = read_img_data(nim, prefer='unscaled')
         header = nim.get_header()
         affine = nim.get_affine()
@@ -124,7 +136,7 @@ def compute_mask_files(input_filename, output_filename=None,
                 mean_volume = vol_arr.mean(axis=-1)
             # Make a copy, to avoid holding a reference on the full array,
             # and thus polluting the memory.
-            first_volume = vol_arr[:,:,:,0].copy()
+            first_volume = vol_arr[..., 0].copy()
         elif vol_arr.ndim == 3:
             mean_volume = first_volume = vol_arr
         else:
@@ -135,7 +147,7 @@ def compute_mask_files(input_filename, output_filename=None,
         if len(list(input_filename)) == 0:
             raise ValueError('input_filename should be a non-empty '
                 'list of file names')
-        # We have several images, we do mean on the fly, 
+        # We have several images, we do mean on the fly,
         # to avoid loading all the data in the memory
         # We do not use the unscaled data here?:
         # if the scalefactor is being used to record real
@@ -150,19 +162,19 @@ def compute_mask_files(input_filename, output_filename=None,
             else:
                 mean_volume += nim.get_data().squeeze()
         mean_volume /= float(len(list(input_filename)))
-        
     del nim
     if np.isnan(mean_volume).any():
         tmp = mean_volume.copy()
         tmp[np.isnan(tmp)] = 0
         mean_volume = tmp
-        
-    mask = compute_mask(mean_volume, first_volume, m, M, cc)
-      
+
+    mask = compute_mask(mean_volume, first_volume, m, M, cc,
+                        opening=opening, exclude_zeros=exclude_zeros)
+
     if output_filename is not None:
         header['descrip'] = 'mask'
-        output_image = nifti1.Nifti1Image(mask.astype(np.uint8), 
-                                            affine=affine, 
+        output_image = nifti1.Nifti1Image(mask.astype(np.uint8),
+                                            affine=affine,
                                             header=header)
         save(output_image, output_filename)
     if not return_mean:
@@ -171,8 +183,8 @@ def compute_mask_files(input_filename, output_filename=None,
         return mask, mean_volume
 
 
-def compute_mask(mean_volume, reference_volume=None, m=0.2, M=0.9, 
-                                                cc=True, opening=True):
+def compute_mask(mean_volume, reference_volume=None, m=0.2, M=0.9,
+                        cc=True, opening=2, exclude_zeros=False):
     """
     Compute a mask file from fMRI data in 3D or 4D ndarrays.
 
@@ -182,13 +194,13 @@ def compute_mask(mean_volume, reference_volume=None, m=0.2, M=0.9,
     m and M of the total image histogram.
 
     In case of failure, it is usually advisable to increase m.
-   
+
     Parameters
     ----------
-    mean_volume : 3D ndarray 
+    mean_volume : 3D ndarray
         mean EPI image, used to compute the threshold for the mask.
     reference_volume: 3D ndarray, optional
-        reference volume used to compute the mask. If none is give, the 
+        reference volume used to compute the mask. If none is give, the
         mean volume is used.
     m : float, optional
         lower fraction of the histogram to be discarded.
@@ -196,98 +208,138 @@ def compute_mask(mean_volume, reference_volume=None, m=0.2, M=0.9,
         upper fraction of the histogram to be discarded.
     cc: boolean, optional
         if cc is True, only the largest connect component is kept.
-    opening: boolean, optional
-        if opening is True, an morphological opening is performed, to keep 
-        only large structures. This step is useful to remove parts of
+    opening: int, optional
+        if opening is larger than 0, an morphological opening is performed,
+        to keep only large structures. This step is useful to remove parts of
         the skull that might have been included.
+    exclude_zeros: boolean, optional
+        Consider zeros as missing values for the computation of the
+        threshold. This option is useful if the images have been
+        resliced with a large padding of zeros.
 
     Returns
     -------
-    mask : 3D boolean ndarray 
+    mask : 3D boolean ndarray
         The brain mask
     """
     if reference_volume is None:
         reference_volume = mean_volume
     sorted_input = np.sort(mean_volume.reshape(-1))
+    if exclude_zeros:
+        sorted_input = sorted_input[sorted_input != 0]
     limiteinf = np.floor(m * len(sorted_input))
     limitesup = np.floor(M * len(sorted_input))
 
     delta = sorted_input[limiteinf + 1:limitesup + 1] \
             - sorted_input[limiteinf:limitesup]
     ia = delta.argmax()
-    threshold = 0.5 * (sorted_input[ia + limiteinf] 
-                        + sorted_input[ia + limiteinf  +1])
-    
+    threshold = 0.5 * (sorted_input[ia + limiteinf]
+                        + sorted_input[ia + limiteinf + 1])
+
     mask = (reference_volume >= threshold)
 
     if cc:
         mask = largest_cc(mask)
-    if opening:
+
+    if opening > 0:
         mask = ndimage.binary_opening(mask.astype(np.int),
-                                        iterations=2)
+                                        iterations=opening)
     return mask.astype(bool)
 
 
-
-def compute_mask_sessions(session_files, m=0.2, M=0.9, cc=1, threshold=0.5):
+def compute_mask_sessions(session_images, m=0.2, M=0.9, cc=1, threshold=0.5,
+                          exclude_zeros=False, return_mean=False, opening=2):
     """ Compute a common mask for several sessions of fMRI data.
 
         Uses the mask-finding algorithmes to extract masks for each
         session, and then keep only the main connected component of the
         a given fraction of the intersection of all the masks.
 
- 
+
     Parameters
     ----------
-    session_files : list of list of strings
-        A list of list of nifti filenames. Each inner list
+    session_images : list of (list of strings) or nipy image objects
+        A list of images/list of nifti filenames. Each inner list/image
         represents a session.
-    threshold : float, optional
-        the inter-session threshold: the fraction of the
-        total number of session in for which a voxel must be in the
-        mask to be kept in the common mask.
-        threshold=1 corresponds to keeping the intersection of all
-        masks, whereas threshold=0 is the union of all masks.
     m : float, optional
         lower fraction of the histogram to be discarded.
     M: float, optional
         upper fraction of the histogram to be discarded.
     cc: boolean, optional
         if cc is True, only the largest connect component is kept.
+    threshold : float, optional
+        the inter-session threshold: the fraction of the
+        total number of session in for which a voxel must be in the
+        mask to be kept in the common mask.
+        threshold=1 corresponds to keeping the intersection of all
+        masks, whereas threshold=0 is the union of all masks.
+    exclude_zeros: boolean, optional
+        Consider zeros as missing values for the computation of the
+        threshold. This option is useful if the images have been
+        resliced with a large padding of zeros.
+    return_mean: boolean, optional
+        if return_mean is True, the mean image accross subjects is
+        returned.
+    opening: int, optional,
+             size of  the morphological opening
 
     Returns
     -------
-    mask : 3D boolean ndarray 
+    mask : 3D boolean ndarray
         The brain mask
+    mean : 3D float array
+        The mean image
     """
-    mask = None
-    for index, session in enumerate(session_files):
-        this_mask = compute_mask_files(session,
-                                       m=m, M=M,
-                                       cc=cc).astype(np.int8)
+    mask, mean = None, None
+    for index, session in enumerate(session_images):
+        if hasattr(session, 'get_data'):
+            mean = session.get_data()
+            if mean.ndim > 3:
+                mean = mean.mean(-1)
+            this_mask = compute_mask(mean, None, m=m, M=M, cc=cc,
+                        opening=opening, exclude_zeros=exclude_zeros)
+            if return_mean:
+                this_mask = this_mask, mean
+        else:
+            this_mask = compute_mask_files(
+                session, m=m, M=M, cc=cc, exclude_zeros=exclude_zeros,
+                return_mean=return_mean, opening=opening)
+        if return_mean:
+            this_mask, this_mean = this_mask
+            if mean is None:
+                mean = this_mean.astype(np.float)
+            else:
+                mean += this_mean
+        this_mask = this_mask.astype(np.int8)
         if mask is None:
             mask = this_mask
         else:
             mask += this_mask
         # Free memory early
         del this_mask
-        
+
     # Take the "half-intersection", i.e. all the voxels that fall within
     # 50% of the individual masks.
-    mask = (mask > threshold*len(list(session_files)))
-   
+    mask = (mask > threshold * len(list(session_images)))
+
     if cc:
         # Select the largest connected component (each mask is
         # connect, but the half-interesection may not be):
         mask = largest_cc(mask)
+    mask = mask.astype(np.bool)
 
-    return mask.astype(np.bool)
+    if return_mean:
+        # Divide by the number of sessions
+        mean /= len(session_images)
+        return mask, mean
+
+    return mask
 
 
 def intersect_masks(input_masks, output_filename=None, threshold=0.5, cc=True):
     """
     Given a list of input mask images, generate the output image which
-    is the the threshold-level intersection of the inputs 
+    is the the threshold-level intersection of the inputs
 
     Parameters
     ----------
@@ -302,15 +354,15 @@ def intersect_masks(input_masks, output_filename=None, threshold=0.5, cc=True):
         masks, whereas threshold=0 is the union of all masks.
     cc: bool, optional
         If true, extract the main connected component
-        
+
     Returns
     -------
     grp_mask, boolean array of shape the image shape
-    """  
+    """
     grp_mask = None
     if threshold > 1:
         raise ValueError('The threshold should be < 1')
-    if threshold <0:
+    if threshold < 0:
         raise ValueError('The threshold should be > 0')
     threshold = min(threshold, 1 - 1.e-7)
 
@@ -322,18 +374,19 @@ def intersect_masks(input_masks, output_filename=None, threshold=0.5, cc=True):
             grp_mask = this_mask.copy().astype(np.int)
         else:
             # If this_mask is floating point and grp_mask is integer, numpy 2
-            # casting rules raise an error for in-place addition. Hence we do it
-            # long-hand. XXX should the masks be coerced to int before addition?
+            # casting rules raise an error for in-place addition.
+            # Hence we do it long-hand.
+            # XXX should the masks be coerced to int before addition?
             grp_mask = grp_mask + this_mask
-    
+
     grp_mask = grp_mask > (threshold * len(list(input_masks)))
-    
+
     if np.any(grp_mask > 0) and cc:
         grp_mask = largest_cc(grp_mask)
-    
+
     if output_filename is not None:
         if isinstance(input_masks[0], basestring):
-            nim = load(input_masks[0]) 
+            nim = load(input_masks[0])
             header = nim.get_header()
             affine = nim.get_affine()
         else:
@@ -349,38 +402,38 @@ def intersect_masks(input_masks, output_filename=None, threshold=0.5, cc=True):
     return grp_mask > 0
 
 
-################################################################################
+###############################################################################
 # Time series extraction
-################################################################################
+###############################################################################
 
 def series_from_mask(filenames, mask, dtype=np.float32,
                      smooth=False, ensure_finite=True):
     """ Read the time series from the given sessions filenames, using the mask.
 
-        Parameters
-        -----------
-        filenames: list of 3D nifti file names, or 4D nifti filename.
-            Files are grouped by session.
-        mask: 3d ndarray
-            3D mask array: true where a voxel should be used.
-        smooth: False or float, optional
+    Parameters
+    -----------
+    filenames: list of 3D nifti file names, or 4D nifti filename.
+               Files are grouped by session.
+    mask: 3d ndarray
+          3D mask array: true where a voxel should be used.
+    smooth: False or float, optional
             If smooth is not False, it gives the size, in voxel of the
             spatial smoothing to apply to the signal.
-        ensure_finite: boolean
+    ensure_finite: boolean, optional
             If ensure_finite is True, the non-finite values (NaNs and infs)
             found in the images will be replaced by zeros
 
-        Returns
-        --------
-        session_series: ndarray
+    Returns
+    --------
+    session_series: ndarray
             3D array of time course: (session, voxel, time)
-        header: header object
+    header: header object
             The header of the first file.
 
-        Notes
-        -----
-        When using smoothing, ensure_finite should be True: as elsewhere non
-        finite values will spread accross the image.
+    Notes
+    -----
+    When using smoothing, ensure_finite should be True: as elsewhere non
+    finite values will spread accross the image.
     """
     assert len(filenames) != 0, (
         'filenames should be a file name or a list of file names, '
@@ -403,7 +456,7 @@ def series_from_mask(filenames, mask, dtype=np.float32,
         if isinstance(series, np.memmap):
             series = np.asarray(series).copy()
         if smooth:
-            vox_size = np.sqrt(np.sum(affine **2, axis=0))
+            vox_size = np.sqrt(np.sum(affine ** 2, axis=0))
             smooth_sigma = smooth / vox_size
             for this_volume in np.rollaxis(series, -1):
                 this_volume[...] = ndimage.gaussian_filter(this_volume,
@@ -421,7 +474,7 @@ def series_from_mask(filenames, mask, dtype=np.float32,
             data = data.astype(dtype)
             if smooth is not False:
                 affine = data_file.get_affine()[:3, :3]
-                vox_size = np.sqrt(np.sum(affine **2, axis=0))
+                vox_size = np.sqrt(np.sum(affine ** 2, axis=0))
                 smooth_sigma = smooth / vox_size
                 data = ndimage.gaussian_filter(data, smooth_sigma)
 
@@ -432,5 +485,3 @@ def series_from_mask(filenames, mask, dtype=np.float32,
                 header = data_file.get_header()
 
     return series, header
-
-
