@@ -1,4 +1,3 @@
-from numpy.testing import Tester
-
+from nipy.testing import Tester
 test = Tester().test
 bench = Tester().bench
