@@ -70,7 +70,7 @@ def surrogate_2d_dataset(n_subj=10, shape=(30, 30), sk=1.0,
         in pixel.
     signal_jitter: float, optionnal
         Random amplitude fluctuation for each activation, added to the
-        amplitude specified by ampli
+        amplitude specified by `ampli`
     width: float or ndarray, optionnal
         Width of the activations
     width_jitter: float
@@ -88,7 +88,7 @@ def surrogate_2d_dataset(n_subj=10, shape=(30, 30), sk=1.0,
     Returns
     -------
     dataset: 3D ndarray
-        The surrogate activation map, with dimensions (n_subj, *shape)
+        The surrogate activation map, with dimensions ``(n_subj,) + shape``
     """
     if seed:
         nr = np.random.RandomState([seed])
@@ -177,8 +177,7 @@ def surrogate_3d_dataset(n_subj=1, shape=(20, 20, 20), mask=None,
     Returns
     -------
     dataset: 3D ndarray
-        The surrogate activation map, with dimensions
-        (n_subj, *shape)
+        The surrogate activation map, with dimensions ``(n_subj,) + shape``
     """
     if seed:
         nr = np.random.RandomState([seed])
@@ -186,7 +185,7 @@ def surrogate_3d_dataset(n_subj=1, shape=(20, 20, 20), mask=None,
         import numpy.random as nr
 
     if mask is not None:
-        shape = mask.get_shape()
+        shape = mask.shape
         mask_data = mask.get_data()
     else:
         mask_data = np.ones(shape)
@@ -276,7 +275,7 @@ def surrogate_4d_dataset(shape=(20, 20, 20), mask=None, n_scans=1, n_sess=1,
         import numpy.random as nr
 
     if mask is not None:
-        shape = mask.get_shape()
+        shape = mask.shape
         affine = mask.get_affine()
         mask_data = mask.get_data().astype('bool')
     else:
