@@ -1,5 +1,7 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
+import tempfile
+
 import numpy as np
 
 from nose import SkipTest
@@ -12,7 +14,7 @@ try:
 except ImportError:
     raise SkipTest('Could not import matplotlib')
 
-from ..activation_maps import demo_plot_map, plot_anat
+from ..activation_maps import demo_plot_map, plot_anat, plot_map
 from ..anat_cache import mni_sform, _AnatCache
 
 
@@ -23,6 +25,8 @@ def test_demo_plot_map():
     import pylab as pl
     pl.switch_backend('svg')
     demo_plot_map()
+    # Test the black background code path
+    demo_plot_map(black_bg=True)
 
 
 def test_plot_anat():
@@ -30,11 +34,24 @@ def test_plot_anat():
     mp.use('svg', warn=False)
     import pylab as pl
     pl.switch_backend('svg')
-    ortho_slicer = plot_anat()
-    data = np.zeros((100, 100, 100))
+    data = np.zeros((20, 20, 20))
     data[3:-3, 3:-3, 3:-3] = 1
+    ortho_slicer = plot_anat(data, mni_sform, dim=True)
+    ortho_slicer = plot_anat(data, mni_sform, cut_coords=(80, -120, -60))
+    # Saving forces a draw, and thus smoke-tests the axes locators
+    pl.savefig(tempfile.TemporaryFile())
     ortho_slicer.edge_map(data, mni_sform, color='c')
 
+    # Test saving with empty plot
+    z_slicer = plot_anat(anat=False, slicer='z')
+    pl.savefig(tempfile.TemporaryFile())
+    z_slicer = plot_anat(slicer='z')
+    pl.savefig(tempfile.TemporaryFile())
+    z_slicer.edge_map(data, mni_sform, color='c')
+    # Smoke test coordinate finder, with and without mask
+    plot_map(np.ma.masked_equal(data, 0), mni_sform, slicer='x')
+    plot_map(data, mni_sform, slicer='y')
+
 
 def test_anat_cache():
     # A smoke test, that can work only if the templates are installed
