@@ -1,4 +1,4 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
-from fmri import FmriImageList, fmri_generator
+from fmri import FmriImageList, axis0_generator
 
