@@ -26,13 +26,10 @@ from nose.tools import assert_true, assert_false, assert_raises
 from numpy.testing import (assert_almost_equal, assert_equal,
                            assert_array_almost_equal)
 
-from nipy.testing import parametric
-
 
 x, y = sympy.symbols(('x', 'y'))
 
 
-@parametric
 def test_implemented_function():
     # Here we check if the default returned functions are anonymous - in
     # the sense that we can have more than one function with the same name
@@ -40,46 +37,45 @@ def test_implemented_function():
     g = implemented_function('f', lambda x: np.sqrt(x))
     l1 = lambdify(x, f(x))
     l2 = lambdify(x, g(x))
-    yield assert_equal(str(f(x)), str(g(x)))
-    yield assert_equal(l1(3), 6)
-    yield assert_equal(l2(3), np.sqrt(3))
+    assert_equal(str(f(x)), str(g(x)))
+    assert_equal(l1(3), 6)
+    assert_equal(l2(3), np.sqrt(3))
     # check that we can pass in a sympy function as input
     func = sympy.Function('myfunc')
-    yield assert_false(hasattr(func, '_imp_'))
+    assert_false(hasattr(func, '_imp_'))
     f = implemented_function(func, lambda x: 2*x)
-    yield assert_true(hasattr(func, '_imp_'))
+    assert_true(hasattr(func, '_imp_'))
 
 
-@parametric
 def test_lambdify():
     # Test lambdify with implemented functions
     # first test basic (sympy) lambdify
     f = sympy.cos
-    yield assert_equal(lambdify(x, f(x))(0), 1)
-    yield assert_equal(lambdify(x, 1 + f(x))(0), 2)
-    yield assert_equal(lambdify((x, y), y + f(x))(0, 1), 2)
+    assert_equal(lambdify(x, f(x))(0), 1)
+    assert_equal(lambdify(x, 1 + f(x))(0), 2)
+    assert_equal(lambdify((x, y), y + f(x))(0, 1), 2)
     # make an implemented function and test
     f = implemented_function("f", lambda x : x+100)
-    yield assert_equal(lambdify(x, f(x))(0), 100)
-    yield assert_equal(lambdify(x, 1 + f(x))(0), 101)
-    yield assert_equal(lambdify((x, y), y + f(x))(0, 1), 101)
+    assert_equal(lambdify(x, f(x))(0), 100)
+    assert_equal(lambdify(x, 1 + f(x))(0), 101)
+    assert_equal(lambdify((x, y), y + f(x))(0, 1), 101)
     # Error for functions with same name and different implementation
     f2 = implemented_function("f", lambda x : x+101)
-    yield assert_raises(ValueError, lambdify, x, f(f2(x)))
+    assert_raises(ValueError, lambdify, x, f(f2(x)))
     # our lambdify, like sympy's lambdify, can also handle tuples,
     # lists, dicts as expressions
     lam = lambdify(x, (f(x), x))
-    yield assert_equal(lam(3), (103, 3))
+    assert_equal(lam(3), (103, 3))
     lam = lambdify(x, [f(x), x])
-    yield assert_equal(lam(3), [103, 3])
+    assert_equal(lam(3), [103, 3])
     lam = lambdify(x, [f(x), (f(x), x)])
-    yield assert_equal(lam(3), [103, (103, 3)])
+    assert_equal(lam(3), [103, (103, 3)])
     lam = lambdify(x, {f(x): x})
-    yield assert_equal(lam(3), {103: 3})
+    assert_equal(lam(3), {103: 3})
     lam = lambdify(x, {f(x): x})
-    yield assert_equal(lam(3), {103: 3})
+    assert_equal(lam(3), {103: 3})
     lam = lambdify(x, {x: f(x)})
-    yield assert_equal(lam(3), {3: 103})
+    assert_equal(lam(3), {3: 103})
 
 
 def gen_BrownianMotion():
@@ -90,7 +86,6 @@ def gen_BrownianMotion():
     return B
 
 
-@parametric
 def test_1d():
     B = gen_BrownianMotion()
     Bs = implemented_function("B", B)
@@ -98,15 +93,14 @@ def test_1d():
     expr = 3*sympy.exp(Bs(t)) + 4
     expected = 3*np.exp(B.y)+4
     ee_vec = lambdify(t, expr, "numpy")
-    yield assert_almost_equal(ee_vec(B.x), expected)
+    assert_almost_equal(ee_vec(B.x), expected)
     # with any arbitrary symbol
     b = sympy.Symbol('b')
     expr = 3*sympy.exp(Bs(b)) + 4
     ee_vec = lambdify(b, expr, "numpy")
-    yield assert_almost_equal(ee_vec(B.x), expected)
+    assert_almost_equal(ee_vec(B.x), expected)
 
 
-@parametric
 def test_2d():
     B1, B2 = [gen_BrownianMotion() for _ in range(2)]
     B1s = implemented_function("B1", B1)
@@ -114,6 +108,4 @@ def test_2d():
     s, t = sympy.symbols(('s', 't'))
     e = B1s(s)+B2s(t)
     ee = lambdify((s,t), e)
-    yield assert_almost_equal(ee(B1.x, B2.x), B1.y + B2.y)
-
-
+    assert_almost_equal(ee(B1.x, B2.x), B1.y + B2.y)
