@@ -1,6 +1,10 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
-import ConfigParser
+try: # python 2
+    from ConfigParser import ConfigParser
+except ImportError: # python 3
+    from configparser import ConfigParser
+
 import os
 
 NIPY_DEFAULTS = dict()
@@ -12,12 +16,12 @@ def get_nipy_info():
     """
     from numpy.distutils.system_info import get_standard_file
     files = get_standard_file('site.cfg')
-    cp = ConfigParser.ConfigParser(NIPY_DEFAULTS)
+    cp = ConfigParser(NIPY_DEFAULTS)
     cp.read(files)
     if not cp.has_section('nipy'):
         cp.add_section('nipy')
     info = dict(cp.items('nipy'))
-    for key, value in info.iteritems():
+    for key, value in info.items():
         if value.startswith('~'):
             info[key] = os.path.expanduser(value)
     # Ugly fix for bug 409269
@@ -41,8 +45,9 @@ def configuration(parent_package='',top_path=None):
     config.add_subpackage('io')
     config.add_subpackage('modalities')
     config.add_subpackage('utils')
-    config.add_subpackage('testing')
+    config.add_subpackage('tests')
     config.add_subpackage('externals')
+    config.add_subpackage('testing')
 
     # Note: this is a special subpackage containing that will later be
     # migrated to whichever parts of the main package they logically
@@ -50,9 +55,6 @@ def configuration(parent_package='',top_path=None):
     # subpackage to make the management and migration easier.
     config.add_subpackage('labs')
 
-    # List all data directories to be loaded here
-    config.add_data_dir('testing')
-
     #####################################################################
     # Store the setup information, including the nipy-specific
     # information in a __config__ file.
@@ -67,6 +69,7 @@ def configuration(parent_package='',top_path=None):
 
     return config
 
+
 if __name__ == '__main__':
     from numpy.distutils.core import setup
     setup(**configuration(top_path='').todict())
