@@ -2,6 +2,8 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
 """Script to auto-generate our API docs.
+
+This script should run in Python 2 and Python 3
 """
 # stdlib imports
 import os
@@ -23,4 +25,4 @@ if __name__ == '__main__':
                                         ]
     docwriter.write_api_docs(outdir)
     docwriter.write_index(outdir, 'gen', relative_to='api')
-    print '%d files written' % len(docwriter.written_modules)
+    print('%d files written' % len(docwriter.written_modules))
